from collections import namedtuple
from dataclasses import dataclass
from itertools import combinations
from typing import List, Tuple, Any, Dict, Iterable, Set, FrozenSet, Optional

from interegular import InvalidSyntax, REFlags
from interegular.fsm import FSM, Alphabet, anything_else
from interegular.patterns import Pattern, Unsupported, parse_pattern
from interegular.utils import logger, soft_repr


@dataclass
class ExampleCollision:
    """
    Captures the full text of an example collision between two regex.
    `main_text` is the part that actually gets captured by the two regex
    `prefix` is the part that is potentially needed for lookbehinds
    `postfix` is the part that is potentially needed for lookahead
    """
    prefix: str
    main_text: str
    postfix: str

    def format_multiline(self, intro: str = "Example Collision: ", indent: str = "",
                         force_pointer: bool = False) -> str:
        """
        Formats this example somewhat similar to a python syntax error.
        - intro is added on the first line
        - indent is added on the second line
        The three parts of the example are concatenated and `^` is used to underline them.

        ExampleCollision(prefix='a', main_text='cd', postfix='ef').format_multiline()

        leads to

        Example Collision: acdef
                             ^^

        This function will escape the character where necessary to stay readable.
        if `force_pointer` is False, the function will not produce the second line if only main_text is set
        """
        if len(intro) < len(indent):
            raise ValueError("Can't have intro be shorter than indent")
        prefix = soft_repr(self.prefix)
        main_text = soft_repr(self.main_text)
        postfix = soft_repr(self.postfix)
        text = f"{prefix}{main_text}{postfix}"
        if len(text) != len(main_text):
            whitespace = ' ' * (len(intro) - len(indent) + len(prefix))
            pointers = '^' * len(main_text)
            return f"{intro}{text}\n{indent}{whitespace}{pointers}"
        else:
            return f"{intro}{text}"

    @property
    def full_text(self):
        return self.prefix + self.main_text + self.postfix


class Comparator:
    """
    A class that represents the main interface for comparing a list of regex to each other.
    It expects a dictionary of arbitrary labels mapped to `Pattern` instances,
    but there is a utility function to create the instances `from_regex` strings.

    The main interface function all expect the abitrary labels to be given, which
    then get mapped to the correct `Pattern` and/or `FSM` instance.

    There is a utility function `mark(a,b)` which allows to mark pairs that shouldn't
    be checked again by `check`.
    """

    def __init__(self, patterns: Dict[Any, Pattern]):
        self._patterns = patterns
        self._marked_pairs: Set[FrozenSet[Any]] = set()
        if not patterns:  # `isdisjoint` can not be called anyway, so we don't need to create a valid state
            return
        self._alphabet = Alphabet.union(*(p.get_alphabet(REFlags(0)) for p in patterns.values()))[0]
        prefix_postfix_s = [p.prefix_postfix for p in patterns.values()]
        self._prefix_postfix = max(p[0] for p in prefix_postfix_s), max(p[1] for p in prefix_postfix_s)
        self._fsms: Dict[Any, FSM] = {}
        self._know_pairs: Dict[Tuple[Any, Any], bool] = {}

    def get_fsm(self, a: Any) -> FSM:
        if a not in self._fsms:
            try:
                self._fsms[a] = self._patterns[a].to_fsm(self._alphabet, self._prefix_postfix)
            except Unsupported as e:
                self._fsms[a] = None
                logger.warning(f"Can't compile Pattern to fsm for {a}\n     {repr(e)}")
            except KeyError:
                self._fsms[a] = None  # In case it was thrown away in `from_regexes`
        return self._fsms[a]

    def isdisjoint(self, a: Any, b: Any) -> bool:
        if (a, b) not in self._know_pairs:
            fa, fb = self.get_fsm(a), self.get_fsm(b)
            if fa is None or fb is None:
                self._know_pairs[a, b] = True  # We can't know. Assume they are disjoint
            else:
                self._know_pairs[a, b] = fa.isdisjoint(fb)
        return self._know_pairs[a, b]

    def check(self, keys: Iterable[Any] = None, skip_marked: bool = False) -> Iterable[Tuple[Any, Any]]:
        if keys is None:
            keys = self._patterns
        for a, b in combinations(keys, 2):
            if skip_marked and self.is_marked(a, b):
                continue
            if not self.isdisjoint(a, b):
                yield a, b

    def get_example_overlap(self, a: Any, b: Any, max_time: float = None) -> ExampleCollision:
        pa, pb = self._patterns[a], self._patterns[b]
        needed_pre = max(pa.prefix_postfix[0], pb.prefix_postfix[0])
        needed_post = max(pa.prefix_postfix[1], pb.prefix_postfix[1])

        # We use the optimal alphabet here instead of the general one since that
        # massively improves performance by every metric.
        alphabet = pa.get_alphabet(REFlags(0)).union(pb.get_alphabet(REFlags(0)))[0]
        fa, fb = pa.to_fsm(alphabet, (needed_pre, needed_post)), pb.to_fsm(alphabet, (needed_pre, needed_post))
        intersection = fa.intersection(fb)
        if max_time is None:
            max_iterations = None
        else:
            # We calculate an approximation for that value of max_iterations
            # that makes sure for this function to finish in under max_time seconds
            # This values will heavily depend on CPU, python version, exact patterns
            # and probably more factors, but this should generally be in the correct
            # ballpark.
            max_iterations = int((max_time - 0.09)/(1.4e-6 * len(alphabet)))
        try:
            text = next(intersection.strings(max_iterations))
        except StopIteration:
            raise ValueError(f"No overlap between {a} and {b} exists")
        text = ''.join(c if c != anything_else else '?' for c in text)
        if needed_post > 0:
            return ExampleCollision(text[:needed_pre], text[needed_pre:-needed_post], text[-needed_post:])
        else:
            return ExampleCollision(text[:needed_pre], text[needed_pre:], '')

    def is_marked(self, a: Any, b: Any) -> bool:
        return frozenset({a, b}) in self._marked_pairs

    @property
    def marked_pairs(self):
        return self._marked_pairs

    def count_marked_pairs(self):
        return len(self._marked_pairs)

    def mark(self, a: Any, b: Any):
        self._marked_pairs.add(frozenset({a, b}))

    @classmethod
    def from_regexes(cls, regexes: Dict[Any, str]):
        patterns = {}
        for k, r in regexes.items():
            try:
                patterns[k] = parse_pattern(r)
            except (Unsupported, InvalidSyntax) as e:
                logger.warning(f"Can't compile regex to Pattern for {k}\n     {repr(e)}")
        return cls(patterns)



from _typeshed import ConvertibleToInt, Incomplete
from typing import ClassVar, Literal, overload

from openpyxl.descriptors.base import Alias, Bool, Integer, String, _ConvertibleToBool
from openpyxl.descriptors.serialisable import Serialisable

class _Protected:
    @overload
    def set_password(self, value: str = "", already_hashed: Literal[False] = False) -> None: ...
    @overload
    def set_password(self, value: str | None, already_hashed: Literal[True]) -> None: ...
    @overload
    def set_password(self, value: str | None = "", *, already_hashed: Literal[True]) -> None: ...
    @property
    def password(self) -> str | None: ...
    @password.setter
    def password(self, value: str) -> None: ...

class SheetProtection(Serialisable, _Protected):
    tagname: ClassVar[str]
    sheet: Bool[Literal[False]]
    enabled: Alias
    objects: Bool[Literal[False]]
    scenarios: Bool[Literal[False]]
    formatCells: Bool[Literal[False]]
    formatColumns: Bool[Literal[False]]
    formatRows: Bool[Literal[False]]
    insertColumns: Bool[Literal[False]]
    insertRows: Bool[Literal[False]]
    insertHyperlinks: Bool[Literal[False]]
    deleteColumns: Bool[Literal[False]]
    deleteRows: Bool[Literal[False]]
    selectLockedCells: Bool[Literal[False]]
    selectUnlockedCells: Bool[Literal[False]]
    sort: Bool[Literal[False]]
    autoFilter: Bool[Literal[False]]
    pivotTables: Bool[Literal[False]]
    saltValue: Incomplete
    spinCount: Integer[Literal[True]]
    algorithmName: String[Literal[True]]
    hashValue: Incomplete
    __attrs__: ClassVar[tuple[str, ...]]
    password: Incomplete
    def __init__(
        self,
        sheet: _ConvertibleToBool = False,
        objects: _ConvertibleToBool = False,
        scenarios: _ConvertibleToBool = False,
        formatCells: _ConvertibleToBool = True,
        formatRows: _ConvertibleToBool = True,
        formatColumns: _ConvertibleToBool = True,
        insertColumns: _ConvertibleToBool = True,
        insertRows: _ConvertibleToBool = True,
        insertHyperlinks: _ConvertibleToBool = True,
        deleteColumns: _ConvertibleToBool = True,
        deleteRows: _ConvertibleToBool = True,
        selectLockedCells: _ConvertibleToBool = False,
        selectUnlockedCells: _ConvertibleToBool = False,
        sort: _ConvertibleToBool = True,
        autoFilter: _ConvertibleToBool = True,
        pivotTables: _ConvertibleToBool = True,
        password=None,
        algorithmName: str | None = None,
        saltValue=None,
        spinCount: ConvertibleToInt | None = None,
        hashValue=None,
    ) -> None: ...
    @overload
    def set_password(self, value: str = "", already_hashed: Literal[False] = False) -> None: ...
    @overload
    def set_password(self, value: str | None, already_hashed: Literal[True]) -> None: ...
    @overload
    def set_password(self, value: str | None = "", *, already_hashed: Literal[True]) -> None: ...
    def enable(self) -> None: ...
    def disable(self) -> None: ...
    def __bool__(self) -> bool: ...



from typing import Generic, TypeVar

from tensorflow._aliases import AnyArray

_Value_co = TypeVar("_Value_co", covariant=True)

class RemoteValue(Generic[_Value_co]):
    def fetch(self) -> AnyArray: ...
    def get(self) -> _Value_co: ...

def __getattr__(name: str): ...  # incomplete module



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["min_maximal_matching"]

@_dispatchable
def min_maximal_matching(G: Graph[_Node]) -> set[Incomplete]: ...



from collections.abc import Iterable, Sequence
from typing import Any, Literal, Protocol, type_check_only

from Xlib._typing import ErrorHandler
from Xlib.protocol import request, rq
from Xlib.protocol.structs import _Arc6IntSequence, _Rectangle4IntSequence, _RGB3IntIterable, _Segment4IntSequence
from Xlib.xobject import colormap, cursor, fontable, resource

# Protocol for the parts of PIL.Image.Image used by python-xlib.
@type_check_only
class _PilImage(Protocol):
    @property
    def mode(self) -> str: ...
    @property
    def size(self) -> tuple[int, int]: ...
    def crop(self, box: tuple[int, int, int, int], /) -> _PilImage: ...
    def tobytes(self, encoder_name: Literal["raw"], rawmode: str, stride: int, x: Literal[0], /) -> bytes: ...

class Drawable(resource.Resource):
    __drawable__ = resource.Resource.__resource__
    def get_geometry(self) -> request.GetGeometry: ...
    def create_pixmap(self, width: int, height: int, depth: int) -> Pixmap: ...
    def create_gc(self, **keys: object) -> fontable.GC: ...
    def copy_area(
        self,
        gc: int,
        src_drawable: int,
        src_x: int,
        src_y: int,
        width: int,
        height: int,
        dst_x: int,
        dst_y: int,
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def copy_plane(
        self,
        gc: int,
        src_drawable: int,
        src_x: int,
        src_y: int,
        width: int,
        height: int,
        dst_x: int,
        dst_y: int,
        bit_plane: int,
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def poly_point(
        self, gc: int, coord_mode: int, points: Sequence[tuple[int, int]], onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def point(self, gc: int, x: int, y: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def poly_line(
        self, gc: int, coord_mode: int, points: Sequence[tuple[int, int]], onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def line(self, gc: int, x1: int, y1: int, x2: int, y2: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def poly_segment(
        self, gc: int, segments: Sequence[_Segment4IntSequence], onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def poly_rectangle(
        self, gc: int, rectangles: Sequence[_Rectangle4IntSequence], onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def rectangle(
        self, gc: int, x: int, y: int, width: int, height: int, onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def poly_arc(self, gc: int, arcs: Sequence[_Arc6IntSequence], onerror: ErrorHandler[object] | None = None) -> None: ...
    def arc(
        self,
        gc: int,
        x: int,
        y: int,
        width: int,
        height: int,
        angle1: int,
        angle2: int,
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def fill_poly(
        self, gc: int, shape: int, coord_mode: int, points: Sequence[tuple[int, int]], onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def poly_fill_rectangle(
        self, gc: int, rectangles: Sequence[_Rectangle4IntSequence], onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def fill_rectangle(
        self, gc: int, x: int, y: int, width: int, height: int, onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def poly_fill_arc(self, gc: int, arcs: Sequence[_Arc6IntSequence], onerror: ErrorHandler[object] | None = None) -> None: ...
    def fill_arc(
        self,
        gc: int,
        x: int,
        y: int,
        width: int,
        height: int,
        angle1: int,
        angle2: int,
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def put_image(
        self,
        gc: int,
        x: int,
        y: int,
        width: int,
        height: int,
        format: int,
        depth: int,
        left_pad: int,
        data: bytes | bytearray,
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def put_pil_image(self, gc: int, x: int, y: int, image: _PilImage, onerror: ErrorHandler[object] | None = None) -> None: ...
    def get_image(self, x: int, y: int, width: int, height: int, format: int, plane_mask: int) -> request.GetImage: ...
    def draw_text(
        self, gc: int, x: int, y: int, text: dict[str, str | int], onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def poly_text(
        self, gc: int, x: int, y: int, items: Sequence[dict[str, str | int]], onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def poly_text_16(
        self, gc: int, x: int, y: int, items: Sequence[dict[str, str | int]], onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def image_text(self, gc: int, x: int, y: int, string: str, onerror: ErrorHandler[object] | None = None) -> None: ...
    def image_text_16(self, gc: int, x: int, y: int, string: str, onerror: ErrorHandler[object] | None = None) -> None: ...
    def query_best_size(self, item_class: int, width: int, height: int) -> request.QueryBestSize: ...

class Window(Drawable):
    __window__ = resource.Resource.__resource__
    def create_window(
        self,
        x: int,
        y: int,
        width: int,
        height: int,
        border_width: int,
        depth: int,
        window_class: int = 0,
        visual: int = 0,
        onerror: ErrorHandler[object] | None = None,
        **keys: object,
    ) -> Window: ...
    def change_attributes(self, onerror: ErrorHandler[object] | None = None, **keys: object) -> None: ...
    def get_attributes(self) -> request.GetWindowAttributes: ...
    def destroy(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def destroy_sub_windows(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def change_save_set(self, mode: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def reparent(self, parent: int, x: int, y: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def map(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def map_sub_windows(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def unmap(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def unmap_sub_windows(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def configure(self, onerror: ErrorHandler[object] | None = None, **keys: object) -> None: ...
    def circulate(self, direction: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def raise_window(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def query_tree(self) -> request.QueryTree: ...
    def change_property(
        self,
        property: int,
        property_type: int,
        format: int,
        data: Sequence[float] | Sequence[str],
        mode: int = 0,
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def change_text_property(
        self, property: int, property_type: int, data: bytes | str, mode: int = 0, onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def delete_property(self, property: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def get_property(
        self, property: int, property_type: int, offset: int, length: int, delete: bool = False
    ) -> request.GetProperty | None: ...
    def get_full_property(self, property: int, property_type: int, sizehint: int = 10) -> request.GetProperty | None: ...
    def get_full_text_property(self, property: int, property_type: int = 0, sizehint: int = 10) -> str | None: ...
    def list_properties(self) -> list[int]: ...
    def set_selection_owner(self, selection: int, time: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def convert_selection(
        self, selection: int, target: int, property: int, time: int, onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def send_event(
        self, event: rq.Event, event_mask: int = 0, propagate: bool = False, onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def grab_pointer(
        self, owner_events: bool, event_mask: int, pointer_mode: int, keyboard_mode: int, confine_to: int, cursor: int, time: int
    ) -> int: ...
    def grab_button(
        self,
        button: int,
        modifiers: int,
        owner_events: bool,
        event_mask: int,
        pointer_mode: int,
        keyboard_mode: int,
        confine_to: int,
        cursor: int,
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def ungrab_button(self, button: int, modifiers: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def grab_keyboard(self, owner_events: bool, pointer_mode: int, keyboard_mode: int, time: int) -> int: ...
    def grab_key(
        self,
        key: int,
        modifiers: int,
        owner_events: bool,
        pointer_mode: int,
        keyboard_mode: int,
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def ungrab_key(self, key: int, modifiers: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def query_pointer(self) -> request.QueryPointer: ...
    def get_motion_events(self, start: int, stop: int) -> rq.Struct: ...
    def translate_coords(self, src_window: int, src_x: int, src_y: int) -> request.TranslateCoords: ...
    def warp_pointer(
        self,
        x: int,
        y: int,
        src_window: int = 0,
        src_x: int = 0,
        src_y: int = 0,
        src_width: int = 0,
        src_height: int = 0,
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def set_input_focus(self, revert_to: int, time: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def clear_area(
        self,
        x: int = 0,
        y: int = 0,
        width: int = 0,
        height: int = 0,
        exposures: bool = False,
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def create_colormap(self, visual: int, alloc: int) -> colormap.Colormap: ...
    def list_installed_colormaps(self) -> list[colormap.Colormap]: ...
    def rotate_properties(self, properties: Sequence[int], delta: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def set_wm_name(self, name: bytes | str, onerror: ErrorHandler[object] | None = None) -> None: ...
    def get_wm_name(self) -> str | None: ...
    def set_wm_icon_name(self, name: bytes | str, onerror: ErrorHandler[object] | None = None) -> None: ...
    def get_wm_icon_name(self) -> str | None: ...
    def set_wm_class(self, inst: str, cls: str, onerror: ErrorHandler[object] | None = None) -> None: ...
    def get_wm_class(self) -> tuple[str, str] | None: ...
    def set_wm_transient_for(self, window: Window, onerror: ErrorHandler[object] | None = None) -> None: ...
    def get_wm_transient_for(self) -> Window | None: ...
    def set_wm_protocols(self, protocols: Iterable[int], onerror: ErrorHandler[object] | None = None) -> None: ...
    def get_wm_protocols(self) -> list[int]: ...
    def set_wm_colormap_windows(self, windows: Iterable[Window], onerror: ErrorHandler[object] | None = None) -> None: ...
    def get_wm_colormap_windows(self) -> Iterable[Window]: ...
    def set_wm_client_machine(self, name: bytes | str, onerror: ErrorHandler[object] | None = None) -> None: ...
    def get_wm_client_machine(self) -> str | None: ...
    def set_wm_normal_hints(
        self, hints: rq.DictWrapper | dict[str, Any] = {}, onerror: ErrorHandler[object] | None = None, **keys: object
    ) -> None: ...
    def get_wm_normal_hints(self) -> rq.DictWrapper | None: ...
    def set_wm_hints(
        self, hints: rq.DictWrapper | dict[str, Any] = {}, onerror: ErrorHandler[object] | None = None, **keys: object
    ) -> None: ...
    def get_wm_hints(self) -> rq.DictWrapper | None: ...
    def set_wm_state(
        self, hints: rq.DictWrapper | dict[str, Any] = {}, onerror: ErrorHandler[object] | None = None, **keys: object
    ) -> None: ...
    def get_wm_state(self) -> rq.DictWrapper | None: ...
    def set_wm_icon_size(
        self, hints: rq.DictWrapper | dict[str, Any] = {}, onerror: ErrorHandler[object] | None = None, **keys: object
    ) -> None: ...
    def get_wm_icon_size(self) -> rq.DictWrapper | None: ...

class Pixmap(Drawable):
    __pixmap__ = resource.Resource.__resource__
    def free(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def create_cursor(
        self, mask: int, foreground: _RGB3IntIterable, background: _RGB3IntIterable, x: int, y: int
    ) -> cursor.Cursor: ...

def roundup(value: int, unit: int) -> int: ...



from _typeshed import Incomplete

class WebhookTestingGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def sample_notification(self, kind: str, id: str, source_merchant_id: str | None = None) -> dict[str, str | bytes]: ...



from _typeshed import Incomplete

from reportlab.pdfbase.pdfdoc import PDFObject

def textFieldAbsolute(canvas, title, x, y, width, height, value: str = "", maxlen: int = 1000000, multiline: int = 0): ...
def textFieldRelative(canvas, title, xR, yR, width, height, value: str = "", maxlen: int = 1000000, multiline: int = 0): ...
def buttonFieldAbsolute(canvas, title, value, x, y, width: float = 16.7704, height: float = 14.907): ...
def buttonFieldRelative(canvas, title, value, xR, yR, width: float = 16.7704, height: float = 14.907): ...
def selectFieldAbsolute(canvas, title, value, options, x, y, width, height) -> None: ...
def selectFieldRelative(canvas, title, value, options, xR, yR, width, height): ...
def getForm(canvas): ...

class AcroForm(PDFObject):
    fields: Incomplete
    def __init__(self) -> None: ...
    def textField(
        self, canvas, title, xmin, ymin, xmax, ymax, value: str = "", maxlen: int = 1000000, multiline: int = 0
    ) -> None: ...
    def selectField(self, canvas, title, value, options, xmin, ymin, xmax, ymax) -> None: ...
    def buttonField(self, canvas, title, value, xmin, ymin, width: float = 16.7704, height: float = 14.907) -> None: ...
    def format(self, document): ...

FormPattern: Incomplete

def FormFontsDictionary(): ...
def FormResources(): ...

ZaDbPattern: Incomplete
FormResourcesDictionaryPattern: Incomplete
FORMFONTNAMES: Incomplete
EncodingPattern: Incomplete
PDFDocEncodingPattern: Incomplete

def FormFont(BaseFont, Name): ...

FormFontPattern: Incomplete

def resetPdfForm() -> None: ...
def TextField(
    title,
    value,
    xmin,
    ymin,
    xmax,
    ymax,
    page,
    maxlen: int = 1000000,
    font: str = "Helvetica-Bold",
    fontsize: int = 9,
    R: int = 0,
    G: int = 0,
    B: float = 0.627,
    multiline: int = 0,
): ...

TextFieldPattern: Incomplete

def SelectField(
    title,
    value,
    options,
    xmin,
    ymin,
    xmax,
    ymax,
    page,
    font: str = "Helvetica-Bold",
    fontsize: int = 9,
    R: int = 0,
    G: int = 0,
    B: float = 0.627,
): ...

SelectFieldPattern: Incomplete

def ButtonField(title, value, xmin, ymin, page, width: float = 16.7704, height: float = 14.907): ...

ButtonFieldPattern: Incomplete

def buttonStreamDictionary(width: float = 16.7704, height: float = 14.907): ...
def ButtonStream(content, width: float = 16.7704, height: float = 14.907): ...



from _typeshed import Incomplete

from antlr4 import DFA as DFA
from antlr4.dfa.DFAState import DFAState as DFAState
from antlr4.Utils import str_list as str_list

class DFASerializer:
    __slots__ = ("dfa", "literalNames", "symbolicNames")
    dfa: Incomplete
    literalNames: list[str] | None
    symbolicNames: list[str] | None
    def __init__(self, dfa: DFA, literalNames: list[str] | None = None, symbolicNames: list[str] | None = None) -> None: ...
    def getEdgeLabel(self, i: int): ...
    def getStateString(self, s: DFAState): ...

class LexerDFASerializer(DFASerializer):
    def __init__(self, dfa: DFA) -> None: ...
    def getEdgeLabel(self, i: int): ...



STATE_ANY: int
STATE_ESCAPE: int
STATE_ESCAPE_HEX: int

def to_dn(
    iterator, decompose: bool = False, remove_space: bool = False, space_around_equal: bool = False, separate_rdn: bool = False
): ...
def parse_dn(dn, escape: bool = False, strip: bool = False): ...
def safe_dn(dn, decompose: bool = False, reverse: bool = False): ...
def safe_rdn(dn, decompose: bool = False): ...
def escape_rdn(rdn: str) -> str: ...



import types
from collections.abc import Callable, Collection, Iterable, Iterator, Mapping
from functools import cached_property
from types import TracebackType
from typing import Any
from typing_extensions import Self, TypeAlias
from urllib.request import Request

from yt_dlp.cache import Cache
from yt_dlp.cookies import YoutubeDLCookieJar
from yt_dlp.networking import RequestDirector, RequestHandler, Response
from yt_dlp.utils import Namespace

from . import _Params
from .extractor.common import InfoExtractor, _InfoDict
from .postprocessor.common import PostProcessor
from .utils._utils import _ProgressState

_FormatSelector: TypeAlias = Callable[[Mapping[str, Any]], Iterator[Any]]

class YoutubeDL:
    params: _Params
    cache: Cache
    format_selector: _FormatSelector
    archive: set[str]
    def __init__(self, params: _Params | None = None, auto_init: bool = True) -> None: ...
    def warn_if_short_id(self, argv: list[str]) -> None: ...
    def add_info_extractor(self, ie: InfoExtractor) -> None: ...
    def get_info_extractor(self, ie_key: str) -> InfoExtractor: ...
    def add_default_info_extractors(self) -> None: ...
    def add_post_processor(self, pp: PostProcessor, when: str = "post_process") -> None: ...
    def add_post_hook(self, ph: Callable[..., object]) -> None: ...
    def add_close_hook(self, ch: Callable[..., object]) -> None: ...
    def add_progress_hook(self, ph: Callable[..., object]) -> None: ...
    def add_postprocessor_hook(self, ph: Callable[..., object]) -> None: ...
    def to_stdout(self, message: str, skip_eol: bool = False, quiet: bool | None = None) -> None: ...
    def to_screen(self, message: str, skip_eol: bool = False, quiet: bool | None = None, only_once: bool = False) -> None: ...
    def to_stderr(self, message: str, only_once: bool = False) -> None: ...
    def to_console_title(
        self, message: str | None = None, progress_state: _ProgressState | None = None, percent: int | None = None
    ) -> None: ...
    def save_console_title(self) -> None: ...
    def restore_console_title(self) -> None: ...
    def __enter__(self) -> Self: ...
    def save_cookies(self) -> None: ...
    def __exit__(self, *args: object) -> None: ...
    def close(self) -> None: ...
    def trouble(self, message: str | None = None, tb: TracebackType | None = None, is_error: bool = True) -> None: ...
    Styles: Namespace
    def report_warning(self, message: str, only_once: bool = False) -> None: ...
    def deprecation_warning(self, message: str, *, stacklevel: int = 0) -> None: ...
    def deprecated_feature(self, message: str) -> None: ...
    def report_error(self, message: str, tb: types.TracebackType | None = None, is_error: bool = True) -> None: ...
    def write_debug(self, message: str, only_once: bool = False) -> None: ...
    def report_file_already_downloaded(self, file_name: str) -> None: ...
    def report_file_delete(self, file_name: str) -> None: ...
    def raise_no_formats(self, info: str, forced: bool = False, *, msg: str | None = None) -> None: ...
    def get_output_path(self, dir_type: str = "", filename: str | None = None) -> str: ...
    @staticmethod
    def escape_outtmpl(outtmpl: str) -> str: ...
    @classmethod
    def validate_outtmpl(cls, outtmpl: str) -> ValueError | None: ...
    def prepare_outtmpl(self, outtmpl: str, info_dict: _InfoDict, sanitize: bool = False) -> tuple[str, dict[str, Any]]: ...
    def evaluate_outtmpl(self, outtmpl: str, info_dict: _InfoDict, sanitize: bool = False) -> str: ...
    def prepare_filename(
        self, info_dict: _InfoDict, dir_type: str = "", *, outtmpl: str | None = None, warn: bool = False
    ) -> str: ...
    @staticmethod
    def add_extra_info(info_dict: _InfoDict, extra_info: Mapping[str, Any]) -> None: ...
    def extract_info(
        self,
        url: str,
        download: bool = True,
        ie_key: str | None = None,
        extra_info: Any | None = None,
        process: bool = True,
        force_generic_extractor: bool = False,
    ) -> _InfoDict: ...
    def add_default_extra_info(self, ie_result: _InfoDict, ie: InfoExtractor, url: str) -> None: ...
    def process_ie_result(
        self, ie_result: _InfoDict, download: bool = True, extra_info: Mapping[str, Any] | None = None
    ) -> _InfoDict: ...
    def build_format_selector(self, format_spec: str) -> _FormatSelector: ...
    def sort_formats(self, info_dict: _InfoDict) -> None: ...
    def process_video_result(self, info_dict: _InfoDict, download: bool = True) -> _InfoDict: ...
    def process_subtitles(
        self, video_id: str, normal_subtitles: Mapping[str, Any], automatic_captions: Mapping[str, Any]
    ) -> dict[str, Any] | None: ...
    def dl(self, name: str, info: _InfoDict, subtitle: bool = False, test: bool = False) -> bool: ...
    def existing_file(self, filepaths: Iterable[str], *, default_overwrite: bool = True) -> str | None: ...
    def process_info(self, info_dict: _InfoDict) -> None: ...
    def download(self, url_list: Collection[str]) -> None: ...
    def download_with_info_file(self, info_filename: str) -> int: ...
    @staticmethod
    def sanitize_info(info_dict: _InfoDict, remove_private_keys: bool = False) -> _InfoDict | None: ...
    @staticmethod
    def filter_requested_info(info_dict: _InfoDict, actually_filter: bool = True) -> _InfoDict | None: ...
    @staticmethod
    def post_extract(info_dict: _InfoDict) -> None: ...
    def run_pp(self, pp: PostProcessor, infodict: _InfoDict) -> _InfoDict: ...
    def run_all_pps(self, key: str, info: _InfoDict, *, additional_pps: Collection[PostProcessor] | None = None) -> _InfoDict: ...
    def pre_process(
        self, ie_info: _InfoDict, key: str = "pre_process", files_to_move: Mapping[str, Any] | None = None
    ) -> tuple[_InfoDict, list[str] | None]: ...
    def post_process(self, filename: str, info: _InfoDict, files_to_move: Mapping[str, Any] | None = None) -> _InfoDict: ...
    def in_download_archive(self, info_dict: _InfoDict) -> bool: ...
    def record_download_archive(self, info_dict: _InfoDict) -> None: ...
    @staticmethod
    def format_resolution(format: Mapping[str, Any], default: str = "unknown") -> str: ...
    def render_formats_table(self, info_dict: _InfoDict) -> str | None: ...
    def render_thumbnails_table(self, info_dict: _InfoDict) -> str | None: ...
    def render_subtitles_table(self, video_id: str, subtitles: Iterable[Mapping[str, Any]]) -> str | None: ...
    def list_formats(self, info_dict: _InfoDict) -> None: ...
    def list_thumbnails(self, info_dict: _InfoDict) -> None: ...
    def list_subtitles(self, video_id: str, subtitles: Iterable[Mapping[str, Any]], name: str = "subtitles") -> None: ...
    def print_debug_header(self) -> None: ...
    @cached_property
    def proxies(self) -> dict[str, Any]: ...
    @cached_property
    def cookiejar(self) -> YoutubeDLCookieJar: ...
    def urlopen(self, req: Request | str) -> Response: ...
    def build_request_director(
        self, handlers: Collection[RequestHandler], preferences: Collection[Any] | None = None
    ) -> RequestDirector: ...
    def encode(self, s: str) -> bytes: ...
    def get_encoding(self) -> str: ...



import re
from _typeshed import Incomplete

from . import base

fullTree: bool
tag_regexp: re.Pattern[str]
comment_type: Incomplete

class DocumentType:
    name: Incomplete
    publicId: Incomplete
    systemId: Incomplete
    def __init__(self, name, publicId, systemId) -> None: ...

class Document:
    def __init__(self) -> None: ...
    def appendChild(self, element) -> None: ...
    @property
    def childNodes(self): ...

def testSerializer(element) -> str: ...
def tostring(element) -> str: ...

class TreeBuilder(base.TreeBuilder):
    documentClass: Incomplete
    doctypeClass: Incomplete
    elementClass: Incomplete
    commentClass: Incomplete
    fragmentClass: Incomplete
    implementation: Incomplete
    namespaceHTMLElements: Incomplete
    def __init__(self, namespaceHTMLElements, fullTree: bool = False): ...
    insertComment: Incomplete
    initial_comments: Incomplete
    doctype: Incomplete
    def reset(self) -> None: ...
    def testSerializer(self, element): ...
    def getDocument(self): ...
    def getFragment(self): ...
    def insertDoctype(self, token) -> None: ...
    def insertCommentInitial(self, data, parent=None) -> None: ...
    def insertCommentMain(self, data, parent=None) -> None: ...
    document: Incomplete
    def insertRoot(self, token) -> None: ...



from typing import ClassVar

import passlib.utils.handlers as uh

class postgres_md5(uh.HasUserContext, uh.StaticHandler):
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    checksum_size: ClassVar[int]

__all__ = ["postgres_md5"]



from _typeshed import Incomplete, SupportsGetItem
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["attribute_mixing_matrix", "attribute_mixing_dict", "degree_mixing_matrix", "degree_mixing_dict", "mixing_dict"]

@_dispatchable
def attribute_mixing_dict(
    G: Graph[_Node], attribute: str, nodes: Iterable[Incomplete] | None = None, normalized: bool = False
) -> dict[Incomplete, Incomplete]: ...
@_dispatchable
def attribute_mixing_matrix(
    G: Graph[_Node],
    attribute: str,
    nodes: Iterable[Incomplete] | None = None,
    mapping: SupportsGetItem[Incomplete, Incomplete] | None = None,
    normalized: bool = True,
): ...
@_dispatchable
def degree_mixing_dict(
    G: Graph[_Node], x: str = "out", y: str = "in", weight: str | None = None, nodes=None, normalized: bool = False
) -> dict[Incomplete, Incomplete]: ...
@_dispatchable
def degree_mixing_matrix(
    G: Graph[_Node],
    x: str = "out",
    y: str = "in",
    weight: str | None = None,
    nodes: Iterable[Incomplete] | None = None,
    normalized: bool = True,
    mapping: SupportsGetItem[Incomplete, Incomplete] | None = None,
): ...
@_dispatchable
def mixing_dict(xy, normalized: bool = False) -> dict[Incomplete, Incomplete]: ...



from _typeshed import SupportsWrite
from array import array
from collections.abc import Callable, Generator, Sequence
from typing import Any, Final, Literal, overload

from .biffh import *
from .book import Book
from .formatting import XF
from .timemachine import *

OBJ_MSO_DEBUG: Final[int]

class MSODrawing(BaseObject): ...
class MSObj(BaseObject): ...
class MSTxo(BaseObject): ...

class Note(BaseObject):
    author: str
    col_hidden: int
    colx: int
    rich_text_runlist: list[tuple[str, int]] | None
    row_hidden: int
    rowx: int
    show: int
    text: str

class Hyperlink(BaseObject):
    frowx: int | None
    lrowx: int | None
    fcolx: int | None
    lcolx: int | None
    type: str | None
    url_or_path: bytes | str | None
    desc: str | None
    target: str | None
    textmark: str | None
    quicktip: str | None

def unpack_RK(rk_str: bytes) -> float: ...

cellty_from_fmtty: Final[dict[int, int]]
ctype_text: Final[dict[int, str]]

class Cell(BaseObject):
    __slots__ = ["ctype", "value", "xf_index"]
    ctype: Literal[0, 1, 2, 3, 4, 5, 6]
    value: str | float
    xf_index: int | None
    def __init__(self, ctype: Literal[0, 1, 2, 3, 4, 5, 6], value: str, xf_index: int | None = None) -> None: ...

empty_cell: Final[Cell]

class Colinfo(BaseObject):
    width: int
    xf_index: int
    hidden: int
    bit1_flag: int
    outline_level: int
    collapsed: int

class Rowinfo(BaseObject):
    __slots__ = (
        "height",
        "has_default_height",
        "outline_level",
        "outline_group_starts_ends",
        "hidden",
        "height_mismatch",
        "has_default_xf_index",
        "xf_index",
        "additional_space_above",
        "additional_space_below",
    )
    height: int | None
    has_default_height: int | None
    outline_level: int | None
    outline_group_starts_ends: int | None
    hidden: int | None
    height_mismatch: int | None
    has_default_xf_index: int | None
    xf_index: int | None
    additional_space_above: int | None
    additional_space_below: int | None
    def __init__(self) -> None: ...
    def __getstate__(self) -> tuple[int | None, ...]: ...
    def __setstate__(self, state: tuple[int | None, ...]) -> None: ...

class Sheet(BaseObject):
    name: str
    book: Book | None
    nrows: int
    ncols: int
    colinfo_map: dict[int, Colinfo]
    rowinfo_map: dict[int, Rowinfo]
    col_label_ranges: list[tuple[int, int, int, int]]
    row_label_ranges: list[tuple[int, int, int, int]]
    merged_cells: list[tuple[int, int, int, int]]
    rich_text_runlist_map: dict[tuple[int, int], list[tuple[int, int]]]
    defcolwidth: float | None
    standardwidth: float | None
    default_row_height: int | None
    default_row_height_mismatch: int | None
    default_row_hidden: int | None
    default_additional_space_above: int | None
    default_additional_space_below: int | None
    visibility: Literal[0, 1, 2]
    gcw: tuple[int, ...]
    hyperlink_list: list[Hyperlink]
    hyperlink_map: dict[tuple[int, int], Hyperlink]
    cell_note_map: dict[tuple[int, int], Note]
    vert_split_pos: int
    horz_split_pos: int
    horz_split_first_visible: int
    vert_split_first_visible: int
    split_active_pane: int
    has_pane_record: int
    horizontal_page_breaks: list[tuple[int, int, int]]
    vertical_page_breaks: list[tuple[int, int, int]]
    biff_version: int
    logfile: SupportsWrite[str]
    bt: array[int]
    bf: array[int]
    number: int
    verbosity: int
    formatting_info: bool
    ragged_rows: bool
    put_cell: Callable[[int, int, int | None, str, int | None], None]
    first_visible_rowx: int
    first_visible_colx: int
    gridline_colour_index: int
    gridline_colour_rgb: tuple[int, int, int] | None
    cooked_page_break_preview_mag_factor: int
    cooked_normal_view_mag_factor: int
    cached_page_break_preview_mag_factor: int
    cached_normal_view_mag_factor: int
    scl_mag_factor: int | None
    utter_max_rows: int
    utter_max_cols: int
    def __init__(self, book: Book, position: int, name: str, number: int) -> None: ...
    def cell(self, rowx: int, colx: int) -> Cell: ...
    def cell_value(self, rowx: int, colx: int) -> str: ...
    def cell_type(self, rowx: int, colx: int) -> int: ...
    def cell_xf_index(self, rowx: int, colx: int) -> int: ...
    def row_len(self, rowx: int) -> int: ...
    def row(self, rowx: int) -> list[Cell]: ...
    @overload
    def __getitem__(self, item: int) -> list[Cell]: ...
    @overload
    def __getitem__(self, item: tuple[int, int]) -> Cell: ...
    def get_rows(self) -> Generator[list[Cell]]: ...
    __iter__ = get_rows
    def row_types(self, rowx: int, start_colx: int = 0, end_colx: int | None = None) -> Sequence[int]: ...
    def row_values(self, rowx: int, start_colx: int = 0, end_colx: int | None = None) -> Sequence[str]: ...
    def row_slice(self, rowx: int, start_colx: int = 0, end_colx: int | None = None) -> list[Cell]: ...
    def col_slice(self, colx: int, start_rowx: int = 0, end_rowx: int | None = None) -> list[Cell]: ...
    def col_values(self, colx: int, start_rowx: int = 0, end_rowx: int | None = None) -> list[str]: ...
    def col_types(self, colx: int, start_rowx: int = 0, end_rowx: int | None = None) -> list[int]: ...
    col = col_slice
    def tidy_dimensions(self) -> None: ...
    def put_cell_ragged(self, rowx: int, colx: int, ctype: int | None, value: str, xf_index: int | None) -> None: ...
    def put_cell_unragged(self, rowx: int, colx: int, ctype: int | None, value: str, xf_index: int | None) -> None: ...
    def read(self, bk: Book) -> Literal[1]: ...
    def string_record_contents(self, data: bytes) -> str | None: ...
    def update_cooked_mag_factors(self) -> None: ...
    def fixed_BIFF2_xfindex(self, cell_attr: bytes, rowx: int, colx: int, true_xfx: int | None = None) -> int: ...
    def insert_new_BIFF20_xf(self, cell_attr: bytes, style: int = 0) -> int: ...
    def fake_XF_from_BIFF20_cell_attr(self, cell_attr: bytes, style: int = 0) -> XF: ...
    def req_fmt_info(self) -> None: ...
    def computed_column_width(self, colx: int) -> float: ...
    def handle_hlink(self, data: bytes) -> None: ...
    def handle_quicktip(self, data: bytes) -> None: ...
    def handle_msodrawingetc(self, recid: Any, data_len: int, data: bytes) -> None: ...
    def handle_obj(self, data: bytes) -> MSObj | None: ...
    def handle_note(self, data: bytes, txos: dict[int, MSTxo]) -> None: ...
    def handle_txo(self, data: bytes) -> MSTxo | None: ...
    def handle_feat11(self, data: bytes) -> None: ...



from .application import ApplicationCommunicator
from .http import HttpCommunicator
from .live import ChannelsLiveServerTestCase
from .websocket import WebsocketCommunicator

__all__ = ["ApplicationCommunicator", "HttpCommunicator", "ChannelsLiveServerTestCase", "WebsocketCommunicator"]



from Xlib.ext import (
    composite as composite,
    damage as damage,
    dpms as dpms,
    ge as ge,
    nvcontrol as nvcontrol,
    randr as randr,
    record as record,
    res as res,
    screensaver as screensaver,
    security as security,
    shape as shape,
    xfixes as xfixes,
    xinerama as xinerama,
    xinput as xinput,
    xtest as xtest,
)

__all__ = [
    "ge",
    "xtest",
    "shape",
    "xinerama",
    "record",
    "composite",
    "randr",
    "xfixes",
    "security",
    "xinput",
    "nvcontrol",
    "damage",
    "dpms",
    "res",
    "screensaver",
]



import re
from _typeshed import Incomplete
from typing import Final

PORT_SPEC: Final[re.Pattern[str]]

def add_port_mapping(port_bindings, internal_port, external) -> None: ...
def add_port(port_bindings, internal_port_range, external_range) -> None: ...
def build_port_bindings(ports) -> dict[Incomplete, Incomplete]: ...
def port_range(start, end, proto, randomly_available_port: bool = False): ...
def split_port(port: object) -> tuple[Incomplete, Incomplete]: ...



from collections.abc import Generator
from typing import Any, Literal
from typing_extensions import Self

from antlr4.Recognizer import Recognizer
from antlr4.tree.Tree import INVALID_INTERVAL as INVALID_INTERVAL, ParseTreeVisitor as ParseTreeVisitor, RuleNode as RuleNode
from antlr4.tree.Trees import Trees as Trees

Parser: None

class RuleContext(RuleNode):
    __slots__ = ("parentCtx", "invokingState")
    EMPTY: RuleContext | None
    parentCtx: RuleContext | None
    invokingState: int
    def __init__(self, parent: RuleContext | None = None, invokingState: int = -1) -> None: ...
    def depth(self) -> int: ...
    def isEmpty(self) -> bool: ...
    def getSourceInterval(self) -> tuple[int | None, int | None]: ...
    def getRuleContext(self) -> Self: ...
    def getPayload(self) -> Self: ...
    def getText(self) -> str: ...
    def getRuleIndex(self) -> Literal[-1]: ...
    def getAltNumber(self) -> Literal[0]: ...
    def setAltNumber(self, altNumber: int) -> None: ...
    def getChild(self, i: int) -> Any: ...
    def getChildCount(self) -> int: ...
    def getChildren(self) -> Generator[Any, None, None]: ...
    def accept(self, visitor: ParseTreeVisitor) -> None: ...
    def toStringTree(self, ruleNames: list[str] | None = None, recog: Recognizer | None = None) -> str: ...
    def toString(self, ruleNames: list[str], stop: RuleContext) -> str: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.tensor_slice_pb2
import tensorflow.core.framework.types_pb2
import tensorflow.core.framework.versions_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class BundleHeaderProto(google.protobuf.message.Message):
    """Protos used in the tensor bundle module (tf/core/util/tensor_bundle/).

    Special header that is associated with a bundle.

    TODO(zongheng,zhifengc): maybe in the future, we can add information about
    which binary produced this checkpoint, timestamp, etc. Sometime, these can be
    valuable debugging information. And if needed, these can be used as defensive
    information ensuring reader (binary version) of the checkpoint and the writer
    (binary version) must match within certain range, etc.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Endianness:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _EndiannessEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BundleHeaderProto._Endianness.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        LITTLE: BundleHeaderProto._Endianness.ValueType  # 0
        BIG: BundleHeaderProto._Endianness.ValueType  # 1

    class Endianness(_Endianness, metaclass=_EndiannessEnumTypeWrapper):
        """An enum indicating the endianness of the platform that produced this
        bundle.  A bundle can only be read by a platform with matching endianness.
        Defaults to LITTLE, as most modern platforms are little-endian.

        Affects the binary tensor data bytes only, not the metadata in protobufs.
        """

    LITTLE: BundleHeaderProto.Endianness.ValueType  # 0
    BIG: BundleHeaderProto.Endianness.ValueType  # 1

    NUM_SHARDS_FIELD_NUMBER: builtins.int
    ENDIANNESS_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    num_shards: builtins.int
    """Number of data files in the bundle."""
    endianness: global___BundleHeaderProto.Endianness.ValueType
    @property
    def version(self) -> tensorflow.core.framework.versions_pb2.VersionDef:
        """Versioning of the tensor bundle format."""

    def __init__(
        self,
        *,
        num_shards: builtins.int | None = ...,
        endianness: global___BundleHeaderProto.Endianness.ValueType | None = ...,
        version: tensorflow.core.framework.versions_pb2.VersionDef | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["version", b"version"]) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["endianness", b"endianness", "num_shards", b"num_shards", "version", b"version"]
    ) -> None: ...

global___BundleHeaderProto = BundleHeaderProto

@typing.final
class BundleEntryProto(google.protobuf.message.Message):
    """Describes the metadata related to a checkpointed tensor."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DTYPE_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    SHARD_ID_FIELD_NUMBER: builtins.int
    OFFSET_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    CRC32C_FIELD_NUMBER: builtins.int
    SLICES_FIELD_NUMBER: builtins.int
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    """The tensor dtype and shape."""
    shard_id: builtins.int
    """The binary content of the tensor lies in:
      File "shard_id": bytes [offset, offset + size).
    """
    offset: builtins.int
    size: builtins.int
    crc32c: builtins.int
    """The CRC32C checksum of the tensor bytes."""
    @property
    def shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto: ...
    @property
    def slices(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        tensorflow.core.framework.tensor_slice_pb2.TensorSliceProto
    ]:
        """Iff present, this entry represents a partitioned tensor.  The previous
        fields are interpreted as follows:

          "dtype", "shape": describe the full tensor.
          "shard_id", "offset", "size", "crc32c": all IGNORED.
             These information for each slice can be looked up in their own
             BundleEntryProto, keyed by each "slice_name".
        """

    def __init__(
        self,
        *,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
        shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
        shard_id: builtins.int | None = ...,
        offset: builtins.int | None = ...,
        size: builtins.int | None = ...,
        crc32c: builtins.int | None = ...,
        slices: collections.abc.Iterable[tensorflow.core.framework.tensor_slice_pb2.TensorSliceProto] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["shape", b"shape"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "crc32c",
            b"crc32c",
            "dtype",
            b"dtype",
            "offset",
            b"offset",
            "shape",
            b"shape",
            "shard_id",
            b"shard_id",
            "size",
            b"size",
            "slices",
            b"slices",
        ],
    ) -> None: ...

global___BundleEntryProto = BundleEntryProto



import datetime
import urllib
import urllib.request
import xml.etree.ElementTree as etree

from yt_dlp.networking import Request

class compat_HTMLParseError(ValueError): ...

def compat_etree_fromstring(text: str) -> etree.Element[str]: ...
def compat_ord(c: str) -> int: ...
def compat_datetime_from_timestamp(timestamp: float) -> datetime.datetime: ...
def compat_expanduser(path: str) -> str: ...
def urllib_req_to_req(urllib_request: urllib.request.Request) -> Request: ...



from _typeshed import Incomplete, SupportsGetItem

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["min_cost_flow_cost", "min_cost_flow", "cost_of_flow", "max_flow_min_cost"]

@_dispatchable
def min_cost_flow_cost(
    G: Graph[_Node], demand: str = "demand", capacity: str = "capacity", weight: str = "weight"
) -> int | float: ...
@_dispatchable
def min_cost_flow(
    G: Graph[_Node], demand: str = "demand", capacity: str = "capacity", weight: str = "weight"
) -> dict[Incomplete, dict[Incomplete, Incomplete]]: ...
@_dispatchable
def cost_of_flow(G: Graph[_Node], flowDict: SupportsGetItem[Incomplete, Incomplete], weight: str = "weight") -> int | float: ...
@_dispatchable
def max_flow_min_cost(
    G: Graph[_Node], s: str, t: str, capacity: str = "capacity", weight: str = "weight"
) -> dict[Incomplete, dict[Incomplete, Incomplete]]: ...



import ast
import xml.dom
from _typeshed import Unused
from collections.abc import Collection, Iterable, Iterator
from types import CodeType
from typing import Literal

from .DesktopEntry import DesktopEntry

DELETED: Literal["Deleted"] = "Deleted"
NO_DISPLAY: Literal["NoDisplay"] = "NoDisplay"
HIDDEN: Literal["Hidden"] = "Hidden"
EMPTY: Literal["Empty"] = "Empty"
NOT_SHOW_IN: Literal["NotShowIn"] = "NotShowIn"
NO_EXEC: Literal["NoExec"] = "NoExec"

class Menu:
    Name: str
    Directory: Menu | None
    Entries: list[str]
    Doc: str
    Filename: str
    Depth: int
    Parent: Menu | None
    NotInXml: bool
    Show: bool
    Visible: int
    AppDirs: list[str]
    DefaultLayout: str | None
    Deleted: bool | None
    Directories: list[str]
    DirectoryDirs: list[Menu]
    Layout: Layout
    MenuEntries: list[MenuEntry | Menu | Separator]
    Moves: list[Move]
    OnlyUnallocated: bool | None
    Rules: list[Rule]
    Submenus: list[Menu]
    def __init__(self) -> None: ...
    def __add__(self, other: Menu) -> Menu: ...
    def __cmp__(self, other: Menu) -> int: ...
    def __lt__(self, other: object) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def getEntries(self, show_hidden: bool = False) -> Iterator[str]: ...
    def getMenuEntry(self, desktopfileid: int, deep: bool = False) -> MenuEntry: ...
    def getMenu(self, path: str) -> Menu: ...
    def getPath(self, org: bool = False, toplevel: bool = False) -> str: ...
    def getName(self) -> str: ...
    def getGenericName(self) -> str: ...
    def getComment(self) -> str: ...
    def getIcon(self) -> str: ...
    def sort(self) -> None: ...
    def addSubmenu(self, newmenu: Menu) -> None: ...
    def merge_inline(self, submenu: Menu) -> None: ...

class Move:
    Old: str
    New: str
    def __init__(self, old: str = "", new: str = "") -> None: ...
    def __cmp__(self, other: Move) -> int: ...

class Layout:
    show_empty: bool
    inline: bool
    inline_limit: int
    inline_header: bool
    inline_alias: bool
    def __init__(
        self,
        show_empty: bool = False,
        inline: bool = False,
        inline_limit: int = 4,
        inline_header: bool = True,
        inline_alias: bool = False,
    ) -> None: ...
    @property
    def order(self) -> list[list[str]]: ...
    @order.setter
    def order(self, order: list[list[str]]) -> None: ...

class Rule:
    TYPE_INCLUDE: Literal[0]
    TYPE_EXCLUDE: Literal[1]
    @classmethod
    def fromFilename(cls, type: Literal[0, 1], filename: str) -> Rule: ...
    Type: Literal[0, 1]
    expression: ast.Expression
    code: CodeType
    def __init__(self, type: Literal[0, 1], expression: str) -> None: ...
    def apply(self, menuentries: Iterable[MenuEntry], run: int) -> Iterable[MenuEntry]: ...

class MenuEntry:
    TYPE_USER: Literal["User"]
    TYPE_SYSTEM: Literal["System"]
    TYPE_BOTH: Literal["Both"]
    DesktopEntry: DesktopEntry
    Show: Literal[True, False, "Deleted", "NoDisplay", "Hidden", "Empty", "NotShowIn", "NoExec"]
    Visible: Literal[1, 0, "Deleted", "NoDisplay", "Hidden", "Empty", "NotShowIn", "NoExec"]
    Original: MenuEntry | None
    Parents: list[Menu]
    Allocated: bool
    Add: bool
    MatchedInclude: bool
    Categories: list[str]
    def __init__(self, filename: str, dir: str = "", prefix: str = "") -> None: ...
    def save(self) -> None: ...
    def getDir(self) -> str: ...
    def getType(self) -> Literal["User", "System", "Both"]: ...
    Filename: str
    Prefix: str
    DesktopFileID: str
    def setAttributes(self, filename: str, dir: str = "", prefix: str = "") -> None: ...
    def updateAttributes(self) -> None: ...
    def __cmp__(self, other: MenuEntry) -> int: ...
    def __lt__(self, other: MenuEntry) -> bool: ...
    def __eq__(self, other: object) -> bool: ...

class Separator:
    Parent: Menu
    Show: bool
    def __init__(self, parent: Menu) -> None: ...

class Header:
    Name: str
    GenericName: str
    Comment: str
    def __init__(self, name: str, generic_name: str, comment: str) -> None: ...

TYPE_DIR: Literal[0] = 0
TYPE_FILE: Literal[1] = 1

class XMLMenuBuilder:
    debug: bool
    def __init__(self, debug: bool = False) -> None: ...
    cache: MenuEntryCache
    def parse(self, filename: str | None = None) -> Menu: ...
    def parse_menu(self, node: xml.dom.Node, filename: str) -> Menu: ...
    def parse_node(self, node: xml.dom.Node, filename: str, parent: Menu | None = None) -> None: ...
    def parse_layout(self, node: xml.dom.Node) -> Layout: ...
    def parse_move(self, node: xml.dom.Node) -> Move: ...
    def parse_rule(self, node: xml.dom.Node) -> Rule: ...
    def parse_bool_op(self, node: xml.dom.Node, operator: ast.And | ast.Or) -> ast.BoolOp | ast.UnaryOp | ast.Compare | None: ...
    def parse_rule_node(self, node: xml.dom.Node) -> ast.BoolOp | ast.UnaryOp | ast.Compare | None: ...
    def parse_app_dir(self, value: str, filename: str, parent: str) -> None: ...
    def parse_default_app_dir(self, filename: str, parent: str) -> None: ...
    def parse_directory_dir(self, value: str, filename: str, parent: str) -> None: ...
    def parse_default_directory_dir(self, filename: str, parent: str) -> None: ...
    def parse_merge_file(self, value: str, child: Menu | MenuEntry, filename: str, parent: str) -> None: ...
    def parse_merge_dir(self, value: str, child: Menu | MenuEntry, filename: str, parent: str) -> None: ...
    def parse_default_merge_dirs(self, child: Menu | MenuEntry, filename: str, parent: str) -> None: ...
    def merge_file(self, filename: str, child: Unused, parent: Menu) -> None: ...
    def parse_legacy_dir(self, dir_: str, prefix: str, filename: str, parent: str) -> None: ...
    def merge_legacy_dir(self, dir_: str, prefix: str, filename: str, parent: str) -> Menu: ...
    def parse_kde_legacy_dirs(self, filename: str, parent: str) -> None: ...
    def post_parse(self, menu: Menu) -> None: ...
    def generate_not_only_allocated(self, menu: Menu) -> None: ...
    def generate_only_allocated(self, menu: Menu) -> None: ...
    def handle_moves(self, menu: Menu) -> None: ...

class MenuEntryCache:
    cacheEntries: dict[str, list[MenuEntry]]
    cache: dict[str, list[MenuEntry]]
    def __init__(self) -> None: ...
    def add_menu_entries(self, dirs: Iterable[str], prefix: str = "", legacy: bool = False) -> None: ...
    def get_menu_entries(self, dirs: Collection[str], legacy: bool = True) -> list[MenuEntry]: ...

def parse(filename: str | None = None, debug: bool = False) -> XMLMenuBuilder: ...



from _typeshed import Incomplete
from collections import OrderedDict
from collections.abc import Iterator
from functools import cached_property
from typing import Any, ClassVar, Literal
from typing_extensions import TypeAlias

from django.core.exceptions import ValidationError
from django.db.models import Model

Dataset: TypeAlias = Incomplete  # tablib.Dataset

class Error:
    error: Exception
    row: dict[str, Any]
    number: int | None
    def __init__(self, error: Exception, row: dict[str, Any] | None = None, number: int | None = None) -> None: ...
    @cached_property
    def traceback(self) -> str: ...

_ImportType: TypeAlias = Literal["update", "new", "delete", "skip", "error", "invalid"]

class RowResult:
    IMPORT_TYPE_UPDATE: ClassVar[Literal["update"]]
    IMPORT_TYPE_NEW: ClassVar[Literal["new"]]
    IMPORT_TYPE_DELETE: ClassVar[Literal["delete"]]
    IMPORT_TYPE_SKIP: ClassVar[Literal["skip"]]
    IMPORT_TYPE_ERROR: ClassVar[Literal["error"]]
    IMPORT_TYPE_INVALID: ClassVar[Literal["invalid"]]
    valid_import_types: frozenset[_ImportType]
    errors: list[Error]
    validation_error: ValidationError | None
    diff: list[str] | None
    import_type: _ImportType
    row_values: dict[str, Any]
    object_id: Any | None
    object_repr: str | None
    instance: Model
    original: Model
    def __init__(self) -> None: ...
    def add_instance_info(self, instance: Model) -> None: ...
    def is_update(self) -> bool: ...
    def is_new(self) -> bool: ...
    def is_delete(self) -> bool: ...
    def is_skip(self) -> bool: ...
    def is_error(self) -> bool: ...
    def is_invalid(self) -> bool: ...
    def is_valid(self) -> bool: ...

class ErrorRow:
    number: int
    errors: list[Error]
    def __init__(self, number: int, errors: list[Error]) -> None: ...

class InvalidRow:
    number: int
    error: ValidationError
    values: tuple[Any, ...]
    error_dict: dict[str, list[str]]
    def __init__(self, number: int, validation_error: ValidationError, values: tuple[Any, ...]) -> None: ...
    @property
    def field_specific_errors(self) -> dict[str, list[str]]: ...
    @property
    def non_field_specific_errors(self) -> list[str]: ...
    @property
    def error_count(self) -> int: ...

class Result:
    base_errors: list[Error]
    diff_headers: list[str]
    rows: list[RowResult]
    invalid_rows: list[InvalidRow]
    error_rows: list[ErrorRow]
    failed_dataset: Dataset
    totals: OrderedDict[_ImportType, int]
    total_rows: int
    def __init__(self) -> None: ...
    def valid_rows(self) -> list[RowResult]: ...
    def append_row_result(self, row_result: RowResult) -> None: ...
    def append_base_error(self, error: Error) -> None: ...
    def add_dataset_headers(self, headers: list[str] | None) -> None: ...
    def append_failed_row(self, row: dict[str, Any], error: Exception) -> None: ...
    def append_invalid_row(self, number: int, row: dict[str, Any], validation_error: ValidationError) -> None: ...
    def append_error_row(self, number: int, row: dict[str, Any], errors: list[Error]) -> None: ...
    def increment_row_result_total(self, row_result: RowResult) -> None: ...
    def row_errors(self) -> list[tuple[int, Any]]: ...
    def has_errors(self) -> bool: ...
    def has_validation_errors(self) -> bool: ...
    def __iter__(self) -> Iterator[RowResult]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
This file is a copy of the TensorBoard ProjectorConfig proto.
Keep this file in sync with the source proto definition at
https://github.com/tensorflow/tensorboard/blob/master/tensorboard/plugins/projector/projector_config.proto
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class SpriteMetadata(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IMAGE_PATH_FIELD_NUMBER: builtins.int
    SINGLE_IMAGE_DIM_FIELD_NUMBER: builtins.int
    image_path: builtins.str
    @property
    def single_image_dim(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """[width, height] of a single image in the sprite."""

    def __init__(
        self, *, image_path: builtins.str | None = ..., single_image_dim: collections.abc.Iterable[builtins.int] | None = ...
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["image_path", b"image_path", "single_image_dim", b"single_image_dim"]
    ) -> None: ...

global___SpriteMetadata = SpriteMetadata

@typing.final
class EmbeddingInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TENSOR_NAME_FIELD_NUMBER: builtins.int
    METADATA_PATH_FIELD_NUMBER: builtins.int
    BOOKMARKS_PATH_FIELD_NUMBER: builtins.int
    TENSOR_SHAPE_FIELD_NUMBER: builtins.int
    SPRITE_FIELD_NUMBER: builtins.int
    TENSOR_PATH_FIELD_NUMBER: builtins.int
    tensor_name: builtins.str
    metadata_path: builtins.str
    bookmarks_path: builtins.str
    tensor_path: builtins.str
    """Path to the TSV file holding the tensor values. If missing, the tensor
    is assumed to be stored in the model checkpoint.
    """
    @property
    def tensor_shape(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """Shape of the 2D tensor [N x D]. If missing, it will be inferred from the
        model checkpoint.
        """

    @property
    def sprite(self) -> global___SpriteMetadata: ...
    def __init__(
        self,
        *,
        tensor_name: builtins.str | None = ...,
        metadata_path: builtins.str | None = ...,
        bookmarks_path: builtins.str | None = ...,
        tensor_shape: collections.abc.Iterable[builtins.int] | None = ...,
        sprite: global___SpriteMetadata | None = ...,
        tensor_path: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["sprite", b"sprite"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "bookmarks_path",
            b"bookmarks_path",
            "metadata_path",
            b"metadata_path",
            "sprite",
            b"sprite",
            "tensor_name",
            b"tensor_name",
            "tensor_path",
            b"tensor_path",
            "tensor_shape",
            b"tensor_shape",
        ],
    ) -> None: ...

global___EmbeddingInfo = EmbeddingInfo

@typing.final
class ProjectorConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_CHECKPOINT_PATH_FIELD_NUMBER: builtins.int
    EMBEDDINGS_FIELD_NUMBER: builtins.int
    MODEL_CHECKPOINT_DIR_FIELD_NUMBER: builtins.int
    model_checkpoint_path: builtins.str
    """Path to the checkpoint file. Use either this or model_checkpoint_dir."""
    model_checkpoint_dir: builtins.str
    """Path to the checkpoint directory. The directory will be scanned for the
    latest checkpoint file.
    """
    @property
    def embeddings(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___EmbeddingInfo]: ...
    def __init__(
        self,
        *,
        model_checkpoint_path: builtins.str | None = ...,
        embeddings: collections.abc.Iterable[global___EmbeddingInfo] | None = ...,
        model_checkpoint_dir: builtins.str | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "embeddings",
            b"embeddings",
            "model_checkpoint_dir",
            b"model_checkpoint_dir",
            "model_checkpoint_path",
            b"model_checkpoint_path",
        ],
    ) -> None: ...

global___ProjectorConfig = ProjectorConfig



from typing import Final

XK_Serbian_dje: Final = 0x6A1
XK_Macedonia_gje: Final = 0x6A2
XK_Cyrillic_io: Final = 0x6A3
XK_Ukrainian_ie: Final = 0x6A4
XK_Ukranian_je: Final = 0x6A4
XK_Macedonia_dse: Final = 0x6A5
XK_Ukrainian_i: Final = 0x6A6
XK_Ukranian_i: Final = 0x6A6
XK_Ukrainian_yi: Final = 0x6A7
XK_Ukranian_yi: Final = 0x6A7
XK_Cyrillic_je: Final = 0x6A8
XK_Serbian_je: Final = 0x6A8
XK_Cyrillic_lje: Final = 0x6A9
XK_Serbian_lje: Final = 0x6A9
XK_Cyrillic_nje: Final = 0x6AA
XK_Serbian_nje: Final = 0x6AA
XK_Serbian_tshe: Final = 0x6AB
XK_Macedonia_kje: Final = 0x6AC
XK_Byelorussian_shortu: Final = 0x6AE
XK_Cyrillic_dzhe: Final = 0x6AF
XK_Serbian_dze: Final = 0x6AF
XK_numerosign: Final = 0x6B0
XK_Serbian_DJE: Final = 0x6B1
XK_Macedonia_GJE: Final = 0x6B2
XK_Cyrillic_IO: Final = 0x6B3
XK_Ukrainian_IE: Final = 0x6B4
XK_Ukranian_JE: Final = 0x6B4
XK_Macedonia_DSE: Final = 0x6B5
XK_Ukrainian_I: Final = 0x6B6
XK_Ukranian_I: Final = 0x6B6
XK_Ukrainian_YI: Final = 0x6B7
XK_Ukranian_YI: Final = 0x6B7
XK_Cyrillic_JE: Final = 0x6B8
XK_Serbian_JE: Final = 0x6B8
XK_Cyrillic_LJE: Final = 0x6B9
XK_Serbian_LJE: Final = 0x6B9
XK_Cyrillic_NJE: Final = 0x6BA
XK_Serbian_NJE: Final = 0x6BA
XK_Serbian_TSHE: Final = 0x6BB
XK_Macedonia_KJE: Final = 0x6BC
XK_Byelorussian_SHORTU: Final = 0x6BE
XK_Cyrillic_DZHE: Final = 0x6BF
XK_Serbian_DZE: Final = 0x6BF
XK_Cyrillic_yu: Final = 0x6C0
XK_Cyrillic_a: Final = 0x6C1
XK_Cyrillic_be: Final = 0x6C2
XK_Cyrillic_tse: Final = 0x6C3
XK_Cyrillic_de: Final = 0x6C4
XK_Cyrillic_ie: Final = 0x6C5
XK_Cyrillic_ef: Final = 0x6C6
XK_Cyrillic_ghe: Final = 0x6C7
XK_Cyrillic_ha: Final = 0x6C8
XK_Cyrillic_i: Final = 0x6C9
XK_Cyrillic_shorti: Final = 0x6CA
XK_Cyrillic_ka: Final = 0x6CB
XK_Cyrillic_el: Final = 0x6CC
XK_Cyrillic_em: Final = 0x6CD
XK_Cyrillic_en: Final = 0x6CE
XK_Cyrillic_o: Final = 0x6CF
XK_Cyrillic_pe: Final = 0x6D0
XK_Cyrillic_ya: Final = 0x6D1
XK_Cyrillic_er: Final = 0x6D2
XK_Cyrillic_es: Final = 0x6D3
XK_Cyrillic_te: Final = 0x6D4
XK_Cyrillic_u: Final = 0x6D5
XK_Cyrillic_zhe: Final = 0x6D6
XK_Cyrillic_ve: Final = 0x6D7
XK_Cyrillic_softsign: Final = 0x6D8
XK_Cyrillic_yeru: Final = 0x6D9
XK_Cyrillic_ze: Final = 0x6DA
XK_Cyrillic_sha: Final = 0x6DB
XK_Cyrillic_e: Final = 0x6DC
XK_Cyrillic_shcha: Final = 0x6DD
XK_Cyrillic_che: Final = 0x6DE
XK_Cyrillic_hardsign: Final = 0x6DF
XK_Cyrillic_YU: Final = 0x6E0
XK_Cyrillic_A: Final = 0x6E1
XK_Cyrillic_BE: Final = 0x6E2
XK_Cyrillic_TSE: Final = 0x6E3
XK_Cyrillic_DE: Final = 0x6E4
XK_Cyrillic_IE: Final = 0x6E5
XK_Cyrillic_EF: Final = 0x6E6
XK_Cyrillic_GHE: Final = 0x6E7
XK_Cyrillic_HA: Final = 0x6E8
XK_Cyrillic_I: Final = 0x6E9
XK_Cyrillic_SHORTI: Final = 0x6EA
XK_Cyrillic_KA: Final = 0x6EB
XK_Cyrillic_EL: Final = 0x6EC
XK_Cyrillic_EM: Final = 0x6ED
XK_Cyrillic_EN: Final = 0x6EE
XK_Cyrillic_O: Final = 0x6EF
XK_Cyrillic_PE: Final = 0x6F0
XK_Cyrillic_YA: Final = 0x6F1
XK_Cyrillic_ER: Final = 0x6F2
XK_Cyrillic_ES: Final = 0x6F3
XK_Cyrillic_TE: Final = 0x6F4
XK_Cyrillic_U: Final = 0x6F5
XK_Cyrillic_ZHE: Final = 0x6F6
XK_Cyrillic_VE: Final = 0x6F7
XK_Cyrillic_SOFTSIGN: Final = 0x6F8
XK_Cyrillic_YERU: Final = 0x6F9
XK_Cyrillic_ZE: Final = 0x6FA
XK_Cyrillic_SHA: Final = 0x6FB
XK_Cyrillic_E: Final = 0x6FC
XK_Cyrillic_SHCHA: Final = 0x6FD
XK_Cyrillic_CHE: Final = 0x6FE
XK_Cyrillic_HARDSIGN: Final = 0x6FF



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.widgetbase import Widget
from reportlab.lib.attrmap import *
from reportlab.lib.validators import *

__version__: Final[str]

class RL_CorpLogo(Widget):
    fillColor: Incomplete
    strokeColor: Incomplete
    strokeWidth: float
    background: Incomplete
    border: Incomplete
    borderWidth: int
    shadow: float
    height: int
    width: int
    x: int
    skewX: int
    showPage: int
    oColors: Incomplete
    pageColors: Incomplete
    prec: Incomplete
    def __init__(self) -> None: ...
    def demo(self): ...
    @staticmethod
    def applyPrec(P, prec): ...
    def draw(self): ...

class RL_CorpLogoReversed(RL_CorpLogo):
    background: Incomplete
    fillColor: Incomplete
    def __init__(self) -> None: ...

class RL_CorpLogoThin(Widget):
    fillColor: Incomplete
    strokeColor: Incomplete
    x: int
    y: int
    height: Incomplete
    width: Incomplete
    def __init__(self) -> None: ...
    def demo(self): ...
    def draw(self): ...

class ReportLabLogo:
    origin: Incomplete
    dimensions: Incomplete
    powered_by: Incomplete
    def __init__(self, atx: int = 0, aty: int = 0, width=180.0, height=108.0, powered_by: int = 0) -> None: ...
    def draw(self, canvas) -> None: ...

class RL_BusinessCard(Widget):
    fillColor: Incomplete
    strokeColor: Incomplete
    altStrokeColor: Incomplete
    x: int
    y: int
    height: Incomplete
    width: Incomplete
    borderWidth: Incomplete
    bleed: Incomplete
    cropMarks: int
    border: int
    name: str
    position: str
    telephone: str
    mobile: str
    fax: str
    email: str
    web: str
    rh_blurb_top: Incomplete
    def __init__(self) -> None: ...
    def demo(self): ...
    def draw(self): ...

def test(formats=["pdf", "eps", "jpg", "gif", "svg"]) -> None: ...



from _typeshed import Incomplete

class FrameworkIntegration:
    expires_in: int
    name: Incomplete
    cache: Incomplete
    def __init__(self, name, cache=None) -> None: ...
    def get_state_data(self, session, state): ...
    def set_state_data(self, session, state, data): ...
    def clear_state_data(self, session, state): ...
    def update_token(self, token, refresh_token=None, access_token=None) -> None: ...
    @staticmethod
    def load_config(oauth, name, params) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _DeploymentMode:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DeploymentModeEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_DeploymentMode.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    DEPLOYMENT_MODE_UNSPECIFIED: _DeploymentMode.ValueType  # 0
    DEPLOYMENT_MODE_COLOCATED: _DeploymentMode.ValueType  # 1
    """tf.data service workers colocate with TF workers."""
    DEPLOYMENT_MODE_REMOTE: _DeploymentMode.ValueType  # 2
    """tf.data service workers run in dedicated tf.data hosts."""
    DEPLOYMENT_MODE_HYBRID: _DeploymentMode.ValueType  # 3
    """tf.data service workers run in colocated TF hosts and dedicated tf.data
    hosts.
    """

class DeploymentMode(_DeploymentMode, metaclass=_DeploymentModeEnumTypeWrapper):
    """tf.data service deployment mode."""

DEPLOYMENT_MODE_UNSPECIFIED: DeploymentMode.ValueType  # 0
DEPLOYMENT_MODE_COLOCATED: DeploymentMode.ValueType  # 1
"""tf.data service workers colocate with TF workers."""
DEPLOYMENT_MODE_REMOTE: DeploymentMode.ValueType  # 2
"""tf.data service workers run in dedicated tf.data hosts."""
DEPLOYMENT_MODE_HYBRID: DeploymentMode.ValueType  # 3
"""tf.data service workers run in colocated TF hosts and dedicated tf.data
hosts.
"""
global___DeploymentMode = DeploymentMode

@typing.final
class ProcessingModeDef(google.protobuf.message.Message):
    """Next tag: 2"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _ShardingPolicy:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ShardingPolicyEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ProcessingModeDef._ShardingPolicy.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        OFF: ProcessingModeDef._ShardingPolicy.ValueType  # 0
        """No sharding will be performed. Each worker produces the entire dataset
        without any sharding. With this mode, the best practice is to shuffle the
        dataset nondeterministically so that workers process the dataset in
        different orders.
        """
        DYNAMIC: ProcessingModeDef._ShardingPolicy.ValueType  # 1
        """The input dataset is dynamically split among workers at runtime. Each
        worker gets the next split when it reads data from the dispatcher. There
        is no fixed sharding with this mode.
        """
        FILE: ProcessingModeDef._ShardingPolicy.ValueType  # 2
        """The following are static sharding policies. The semantics are similar to
        `tf.data.experimental.AutoShardPolicy`. These policies require:
        * The tf.data service cluster has a fixed size, and you need to specify
          the workers in DispatcherConfig.
        * Each client only reads from the local tf.data service worker.

        Shards by input files (each worker will get a set of files to process).
        When this option is selected, make sure that there is at least as many
        files as workers. If there are fewer input files than workers, a runtime
        error will be raised.
        """
        DATA: ProcessingModeDef._ShardingPolicy.ValueType  # 3
        """Shards by elements produced by the dataset. Each worker will process the
        whole dataset and discard the portion that is not for itself. Note that
        for this mode to correctly partitions the dataset elements, the dataset
        needs to produce elements in a deterministic order.
        """
        FILE_OR_DATA: ProcessingModeDef._ShardingPolicy.ValueType  # 4
        """Attempts FILE-based sharding, falling back to DATA-based sharding on
        failures.
        """
        HINT: ProcessingModeDef._ShardingPolicy.ValueType  # 5
        """Looks for the presence of `shard(SHARD_HINT, ...)` which is treated as a
        placeholder to replace with `shard(num_workers, worker_index)`.
        """

    class ShardingPolicy(_ShardingPolicy, metaclass=_ShardingPolicyEnumTypeWrapper):
        """Specifies how data is sharded among tf.data service workers."""

    OFF: ProcessingModeDef.ShardingPolicy.ValueType  # 0
    """No sharding will be performed. Each worker produces the entire dataset
    without any sharding. With this mode, the best practice is to shuffle the
    dataset nondeterministically so that workers process the dataset in
    different orders.
    """
    DYNAMIC: ProcessingModeDef.ShardingPolicy.ValueType  # 1
    """The input dataset is dynamically split among workers at runtime. Each
    worker gets the next split when it reads data from the dispatcher. There
    is no fixed sharding with this mode.
    """
    FILE: ProcessingModeDef.ShardingPolicy.ValueType  # 2
    """The following are static sharding policies. The semantics are similar to
    `tf.data.experimental.AutoShardPolicy`. These policies require:
    * The tf.data service cluster has a fixed size, and you need to specify
      the workers in DispatcherConfig.
    * Each client only reads from the local tf.data service worker.

    Shards by input files (each worker will get a set of files to process).
    When this option is selected, make sure that there is at least as many
    files as workers. If there are fewer input files than workers, a runtime
    error will be raised.
    """
    DATA: ProcessingModeDef.ShardingPolicy.ValueType  # 3
    """Shards by elements produced by the dataset. Each worker will process the
    whole dataset and discard the portion that is not for itself. Note that
    for this mode to correctly partitions the dataset elements, the dataset
    needs to produce elements in a deterministic order.
    """
    FILE_OR_DATA: ProcessingModeDef.ShardingPolicy.ValueType  # 4
    """Attempts FILE-based sharding, falling back to DATA-based sharding on
    failures.
    """
    HINT: ProcessingModeDef.ShardingPolicy.ValueType  # 5
    """Looks for the presence of `shard(SHARD_HINT, ...)` which is treated as a
    placeholder to replace with `shard(num_workers, worker_index)`.
    """

    SHARDING_POLICY_FIELD_NUMBER: builtins.int
    sharding_policy: global___ProcessingModeDef.ShardingPolicy.ValueType
    def __init__(self, *, sharding_policy: global___ProcessingModeDef.ShardingPolicy.ValueType | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["sharding_policy", b"sharding_policy"]) -> None: ...

global___ProcessingModeDef = ProcessingModeDef

@typing.final
class DataServiceMetadata(google.protobuf.message.Message):
    """Metadata related to tf.data service datasets.
    Next tag: 4
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Compression:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _CompressionEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[DataServiceMetadata._Compression.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        COMPRESSION_UNSPECIFIED: DataServiceMetadata._Compression.ValueType  # 0
        COMPRESSION_OFF: DataServiceMetadata._Compression.ValueType  # 1
        """No compression."""
        COMPRESSION_SNAPPY: DataServiceMetadata._Compression.ValueType  # 2
        """Snappy compression as defined in tensorflow/core/platform/snappy.h."""

    class Compression(_Compression, metaclass=_CompressionEnumTypeWrapper): ...
    COMPRESSION_UNSPECIFIED: DataServiceMetadata.Compression.ValueType  # 0
    COMPRESSION_OFF: DataServiceMetadata.Compression.ValueType  # 1
    """No compression."""
    COMPRESSION_SNAPPY: DataServiceMetadata.Compression.ValueType  # 2
    """Snappy compression as defined in tensorflow/core/platform/snappy.h."""

    ELEMENT_SPEC_FIELD_NUMBER: builtins.int
    COMPRESSION_FIELD_NUMBER: builtins.int
    CARDINALITY_FIELD_NUMBER: builtins.int
    element_spec: builtins.bytes
    """Serialized element spec."""
    compression: global___DataServiceMetadata.Compression.ValueType
    cardinality: builtins.int
    """Cardinality of the dataset."""
    def __init__(
        self,
        *,
        element_spec: builtins.bytes | None = ...,
        compression: global___DataServiceMetadata.Compression.ValueType | None = ...,
        cardinality: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["element_spec", b"element_spec", "optional_element_spec", b"optional_element_spec"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "cardinality",
            b"cardinality",
            "compression",
            b"compression",
            "element_spec",
            b"element_spec",
            "optional_element_spec",
            b"optional_element_spec",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_element_spec", b"optional_element_spec"]
    ) -> typing.Literal["element_spec"] | None: ...

global___DataServiceMetadata = DataServiceMetadata

@typing.final
class CrossTrainerCacheOptions(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TRAINER_ID_FIELD_NUMBER: builtins.int
    trainer_id: builtins.str
    def __init__(self, *, trainer_id: builtins.str | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["trainer_id", b"trainer_id"]) -> None: ...

global___CrossTrainerCacheOptions = CrossTrainerCacheOptions

@typing.final
class DataServiceConfig(google.protobuf.message.Message):
    """Data service config available to the client through GetDataServiceConfig RPC.
    Next tag: 2
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEPLOYMENT_MODE_FIELD_NUMBER: builtins.int
    deployment_mode: global___DeploymentMode.ValueType
    def __init__(self, *, deployment_mode: global___DeploymentMode.ValueType | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["deployment_mode", b"deployment_mode"]) -> None: ...

global___DataServiceConfig = DataServiceConfig



import datetime
from collections.abc import Iterable

from ..core import ChineseNewYearCalendar

class Taiwan(ChineseNewYearCalendar):
    def is_working_day(
        self,
        day: datetime.date | datetime.datetime,
        extra_working_days: Iterable[datetime.date | datetime.datetime] | None = None,
        extra_holidays: Iterable[datetime.date | datetime.datetime] | None = None,
    ) -> bool: ...



import logging
from _typeshed import Incomplete
from collections.abc import Callable, Sequence
from contextlib import _GeneratorContextManager
from typing import Any, TypeVar, overload

from ..std import tqdm as std_tqdm

_TqdmT = TypeVar("_TqdmT", bound=std_tqdm[Any])

def logging_redirect_tqdm(
    loggers: Sequence[logging.Logger] | None = None, tqdm_class: type[std_tqdm[Any]] = ...
) -> _GeneratorContextManager[None]: ...

# TODO: type *args, **kwargs here more precisely
@overload
def tqdm_logging_redirect(*args, tqdm_class: Callable[..., _TqdmT], **kwargs) -> _GeneratorContextManager[_TqdmT]: ...
@overload
def tqdm_logging_redirect(*args, **kwargs) -> _GeneratorContextManager[std_tqdm[Incomplete]]: ...



from pathlib import Path

from .enums import Align as Align, TextMode as TextMode, XPos as XPos, YPos as YPos
from .fonts import FontFace as FontFace, TextStyle as TextStyle
from .fpdf import FPDF as FPDF, FPDFException as FPDFException, TitleStyle as TitleStyle
from .html import HTML2FPDF as HTML2FPDF, HTMLMixin as HTMLMixin
from .prefs import ViewerPreferences as ViewerPreferences
from .template import FlexTemplate as FlexTemplate, Template as Template
from .util import get_scale_factor as get_scale_factor

__license__: str
__version__: str
FPDF_VERSION: str
FPDF_FONT_DIR: Path

__all__ = [
    "__version__",
    "__license__",
    "FPDF",
    "FPDFException",
    "FontFace",
    "Align",
    "TextMode",
    "XPos",
    "YPos",
    "Template",
    "FlexTemplate",
    "TitleStyle",
    "TextStyle",
    "ViewerPreferences",
    "HTMLMixin",
    "HTML2FPDF",
    "FPDF_VERSION",
    "FPDF_FONT_DIR",
    "get_scale_factor",
]



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["rooted_tree_isomorphism", "tree_isomorphism"]

@_dispatchable
def root_trees(t1, root1, t2, root2): ...
@_dispatchable
def rooted_tree_isomorphism(t1, root1, t2, root2) -> list[tuple[Incomplete, Incomplete]]: ...
@_dispatchable
def tree_isomorphism(t1: Graph[_Node], t2: Graph[_Node]) -> list[tuple[Incomplete, Incomplete]]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _VariableSynchronization:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _VariableSynchronizationEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_VariableSynchronization.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    VARIABLE_SYNCHRONIZATION_AUTO: _VariableSynchronization.ValueType  # 0
    """`AUTO`: Indicates that the synchronization will be determined by the
    current `DistributionStrategy` (eg. With `MirroredStrategy` this would be
    `ON_WRITE`).
    """
    VARIABLE_SYNCHRONIZATION_NONE: _VariableSynchronization.ValueType  # 1
    """`NONE`: Indicates that there will only be one copy of the variable, so
    there is no need to sync.
    """
    VARIABLE_SYNCHRONIZATION_ON_WRITE: _VariableSynchronization.ValueType  # 2
    """`ON_WRITE`: Indicates that the variable will be updated across devices
    every time it is written.
    """
    VARIABLE_SYNCHRONIZATION_ON_READ: _VariableSynchronization.ValueType  # 3
    """`ON_READ`: Indicates that the variable will be aggregated across devices
    when it is read (eg. when checkpointing or when evaluating an op that uses
    the variable).
    """

class VariableSynchronization(_VariableSynchronization, metaclass=_VariableSynchronizationEnumTypeWrapper):
    """Indicates when a distributed variable will be synced."""

VARIABLE_SYNCHRONIZATION_AUTO: VariableSynchronization.ValueType  # 0
"""`AUTO`: Indicates that the synchronization will be determined by the
current `DistributionStrategy` (eg. With `MirroredStrategy` this would be
`ON_WRITE`).
"""
VARIABLE_SYNCHRONIZATION_NONE: VariableSynchronization.ValueType  # 1
"""`NONE`: Indicates that there will only be one copy of the variable, so
there is no need to sync.
"""
VARIABLE_SYNCHRONIZATION_ON_WRITE: VariableSynchronization.ValueType  # 2
"""`ON_WRITE`: Indicates that the variable will be updated across devices
every time it is written.
"""
VARIABLE_SYNCHRONIZATION_ON_READ: VariableSynchronization.ValueType  # 3
"""`ON_READ`: Indicates that the variable will be aggregated across devices
when it is read (eg. when checkpointing or when evaluating an op that uses
the variable).
"""
global___VariableSynchronization = VariableSynchronization

class _VariableAggregation:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _VariableAggregationEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_VariableAggregation.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    VARIABLE_AGGREGATION_NONE: _VariableAggregation.ValueType  # 0
    """`NONE`: This is the default, giving an error if you use a
    variable-update operation with multiple replicas.
    """
    VARIABLE_AGGREGATION_SUM: _VariableAggregation.ValueType  # 1
    """`SUM`: Add the updates across replicas."""
    VARIABLE_AGGREGATION_MEAN: _VariableAggregation.ValueType  # 2
    """`MEAN`: Take the arithmetic mean ("average") of the updates across
    replicas.
    """
    VARIABLE_AGGREGATION_ONLY_FIRST_REPLICA: _VariableAggregation.ValueType  # 3
    """`ONLY_FIRST_REPLICA`: This is for when every replica is performing the same
    update, but we only want to perform the update once. Used, e.g., for the
    global step counter.
    """

class VariableAggregation(_VariableAggregation, metaclass=_VariableAggregationEnumTypeWrapper):
    """Indicates how a distributed variable will be aggregated."""

VARIABLE_AGGREGATION_NONE: VariableAggregation.ValueType  # 0
"""`NONE`: This is the default, giving an error if you use a
variable-update operation with multiple replicas.
"""
VARIABLE_AGGREGATION_SUM: VariableAggregation.ValueType  # 1
"""`SUM`: Add the updates across replicas."""
VARIABLE_AGGREGATION_MEAN: VariableAggregation.ValueType  # 2
"""`MEAN`: Take the arithmetic mean ("average") of the updates across
replicas.
"""
VARIABLE_AGGREGATION_ONLY_FIRST_REPLICA: VariableAggregation.ValueType  # 3
"""`ONLY_FIRST_REPLICA`: This is for when every replica is performing the same
update, but we only want to perform the update once. Used, e.g., for the
global step counter.
"""
global___VariableAggregation = VariableAggregation

@typing.final
class VariableDef(google.protobuf.message.Message):
    """Protocol buffer representing a Variable."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VARIABLE_NAME_FIELD_NUMBER: builtins.int
    INITIAL_VALUE_NAME_FIELD_NUMBER: builtins.int
    INITIALIZER_NAME_FIELD_NUMBER: builtins.int
    SNAPSHOT_NAME_FIELD_NUMBER: builtins.int
    SAVE_SLICE_INFO_DEF_FIELD_NUMBER: builtins.int
    IS_RESOURCE_FIELD_NUMBER: builtins.int
    TRAINABLE_FIELD_NUMBER: builtins.int
    SYNCHRONIZATION_FIELD_NUMBER: builtins.int
    AGGREGATION_FIELD_NUMBER: builtins.int
    variable_name: builtins.str
    """Name of the variable tensor."""
    initial_value_name: builtins.str
    """Name of the tensor holding the variable's initial value."""
    initializer_name: builtins.str
    """Name of the initializer op."""
    snapshot_name: builtins.str
    """Name of the snapshot tensor."""
    is_resource: builtins.bool
    """Whether to represent this as a ResourceVariable."""
    trainable: builtins.bool
    """Whether this variable should be trained."""
    synchronization: global___VariableSynchronization.ValueType
    """Indicates when a distributed variable will be synced."""
    aggregation: global___VariableAggregation.ValueType
    """Indicates how a distributed variable will be aggregated."""
    @property
    def save_slice_info_def(self) -> global___SaveSliceInfoDef:
        """Support for saving variables as slices of a larger variable."""

    def __init__(
        self,
        *,
        variable_name: builtins.str | None = ...,
        initial_value_name: builtins.str | None = ...,
        initializer_name: builtins.str | None = ...,
        snapshot_name: builtins.str | None = ...,
        save_slice_info_def: global___SaveSliceInfoDef | None = ...,
        is_resource: builtins.bool | None = ...,
        trainable: builtins.bool | None = ...,
        synchronization: global___VariableSynchronization.ValueType | None = ...,
        aggregation: global___VariableAggregation.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["save_slice_info_def", b"save_slice_info_def"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "aggregation",
            b"aggregation",
            "initial_value_name",
            b"initial_value_name",
            "initializer_name",
            b"initializer_name",
            "is_resource",
            b"is_resource",
            "save_slice_info_def",
            b"save_slice_info_def",
            "snapshot_name",
            b"snapshot_name",
            "synchronization",
            b"synchronization",
            "trainable",
            b"trainable",
            "variable_name",
            b"variable_name",
        ],
    ) -> None: ...

global___VariableDef = VariableDef

@typing.final
class SaveSliceInfoDef(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FULL_NAME_FIELD_NUMBER: builtins.int
    FULL_SHAPE_FIELD_NUMBER: builtins.int
    VAR_OFFSET_FIELD_NUMBER: builtins.int
    VAR_SHAPE_FIELD_NUMBER: builtins.int
    full_name: builtins.str
    """Name of the full variable of which this is a slice."""
    @property
    def full_shape(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """Shape of the full variable."""

    @property
    def var_offset(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """Offset of this variable into the full variable."""

    @property
    def var_shape(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """Shape of this variable."""

    def __init__(
        self,
        *,
        full_name: builtins.str | None = ...,
        full_shape: collections.abc.Iterable[builtins.int] | None = ...,
        var_offset: collections.abc.Iterable[builtins.int] | None = ...,
        var_shape: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "full_name", b"full_name", "full_shape", b"full_shape", "var_offset", b"var_offset", "var_shape", b"var_shape"
        ],
    ) -> None: ...

global___SaveSliceInfoDef = SaveSliceInfoDef



"""Test cases for psutil.process_iter and its cache_clear method."""

from __future__ import annotations

import psutil

# Test that process_iter can be called as a function
for proc in psutil.process_iter():
    break

# Test that process_iter has cache_clear method
psutil.process_iter.cache_clear()

# Test that cache_clear is callable
clear_method = psutil.process_iter.cache_clear
clear_method()



import datetime
from typing import Final, overload

from pymeeus.Angle import Angle
from pymeeus.Epoch import Epoch

NUTATION_ARG_TABLE: Final[list[list[int]]]
NUTATION_SINE_COEF_TABLE: Final[list[list[float]]]
NUTATION_COSINE_COEF_TABLE: Final[list[list[float]]]

@overload
def mean_obliquity(
    a: Epoch | list[float] | tuple[float, ...] | datetime.date,
    /,
    *,
    leap_seconds: float = 0,
    local: bool = False,
    utc: bool = False,
) -> Angle: ...
@overload
def mean_obliquity(
    year: int, month: int, day: int, /, *, leap_seconds: float = 0, local: bool = False, utc: bool = False
) -> Angle: ...
@overload
def true_obliquity(
    a: Epoch | list[float] | tuple[float, ...] | datetime.date,
    /,
    *,
    leap_seconds: float = 0,
    local: bool = False,
    utc: bool = False,
) -> Angle: ...
@overload
def true_obliquity(
    year: int, month: int, day: int, /, *, leap_seconds: float = 0, local: bool = False, utc: bool = False
) -> Angle: ...
@overload
def nutation_longitude(
    a: Epoch | list[float] | tuple[float, ...] | datetime.date,
    /,
    *,
    leap_seconds: float = 0,
    local: bool = False,
    utc: bool = False,
) -> Angle: ...
@overload
def nutation_longitude(
    year: int, month: int, day: int, /, *, leap_seconds: float = 0, local: bool = False, utc: bool = False
) -> Angle: ...
@overload
def nutation_obliquity(
    a: Epoch | list[float] | tuple[float, ...] | datetime.date,
    /,
    *,
    leap_seconds: float = 0,
    local: bool = False,
    utc: bool = False,
) -> Angle: ...
@overload
def nutation_obliquity(
    year: int, month: int, day: int, /, *, leap_seconds: float = 0, local: bool = False, utc: bool = False
) -> Angle: ...
def precession_equatorial(
    start_epoch: Epoch,
    final_epoch: Epoch,
    start_ra: Angle,
    start_dec: Angle,
    p_motion_ra: float | Angle = 0.0,
    p_motion_dec: float | Angle = 0.0,
) -> tuple[Angle, Angle]: ...
def precession_ecliptical(
    start_epoch: Epoch,
    final_epoch: Epoch,
    start_lon: Angle,
    start_lat: Angle,
    p_motion_lon: float | Angle = 0.0,
    p_motion_lat: float | Angle = 0.0,
) -> tuple[Angle, Angle]: ...
def p_motion_equa2eclip(
    p_motion_ra: Angle, p_motion_dec: Angle, ra: Angle, dec: Angle, lat: Angle, epsilon: Angle
) -> tuple[float, float]: ...
def precession_newcomb(
    start_epoch: Epoch,
    final_epoch: Epoch,
    start_ra: Angle,
    start_dec: Angle,
    p_motion_ra: float | Angle = 0.0,
    p_motion_dec: float | Angle = 0.0,
) -> tuple[Angle, Angle]: ...
def motion_in_space(
    start_ra: Angle,
    start_dec: Angle,
    distance: float,
    velocity: float,
    p_motion_ra: float | Angle,
    p_motion_dec: float | Angle,
    time: float,
) -> tuple[Angle, Angle]: ...
def equatorial2ecliptical(right_ascension: Angle, declination: Angle, obliquity: Angle) -> tuple[Angle, Angle]: ...
def ecliptical2equatorial(longitude: Angle, latitude: Angle, obliquity: Angle) -> tuple[Angle, Angle]: ...
def equatorial2horizontal(hour_angle: Angle, declination: Angle, geo_latitude: Angle) -> tuple[Angle, Angle]: ...
def horizontal2equatorial(azimuth: Angle, elevation: Angle, geo_latitude: Angle) -> tuple[Angle, Angle]: ...
def equatorial2galactic(right_ascension: Angle, declination: Angle) -> tuple[Angle, Angle]: ...
def galactic2equatorial(longitude: Angle, latitude: Angle) -> tuple[Angle, Angle]: ...
def parallactic_angle(hour_angle: Angle, declination: Angle, geo_latitude: Angle) -> Angle | None: ...
def ecliptic_horizon(local_sidereal_time: Angle, geo_latitude: Angle, obliquity: Angle) -> tuple[Angle, Angle, Angle]: ...
def ecliptic_equator(longitude: Angle, latitude: Angle, obliquity: Angle) -> Angle: ...
def diurnal_path_horizon(declination: Angle, geo_latitude: Angle) -> Angle: ...
def times_rise_transit_set(
    longitude: Angle,
    latitude: Angle,
    alpha1: Angle,
    delta1: Angle,
    alpha2: Angle,
    delta2: Angle,
    alpha3: Angle,
    delta3: Angle,
    h0: Angle,
    delta_t: float,
    theta0: Angle,
) -> tuple[float, float, float] | tuple[None, None, None]: ...
def refraction_apparent2true(apparent_elevation: Angle, pressure: float = 1010.0, temperature: float = 10.0) -> Angle: ...
def refraction_true2apparent(true_elevation: Angle, pressure: float = 1010.0, temperature: float = 10.0) -> Angle: ...
def angular_separation(alpha1: Angle, delta1: Angle, alpha2: Angle, delta2: Angle) -> Angle: ...
def minimum_angular_separation(
    alpha1_1: Angle,
    delta1_1: Angle,
    alpha1_2: Angle,
    delta1_2: Angle,
    alpha1_3: Angle,
    delta1_3: Angle,
    alpha2_1: Angle,
    delta2_1: Angle,
    alpha2_2: Angle,
    delta2_2: Angle,
    alpha2_3: Angle,
    delta2_3: Angle,
) -> tuple[float, Angle]: ...
def relative_position_angle(alpha1: Angle, delta1: Angle, alpha2: Angle, delta2: Angle) -> Angle: ...
def planetary_conjunction(
    alpha1_list: list[Angle] | tuple[Angle, ...],
    delta1_list: list[Angle] | tuple[Angle, ...],
    alpha2_list: list[Angle] | tuple[Angle, ...],
    delta2_list: list[Angle] | tuple[Angle, ...],
) -> tuple[float, Angle]: ...
def planet_star_conjunction(
    alpha_list: list[Angle] | tuple[Angle, ...], delta_list: list[Angle] | tuple[Angle, ...], alpha_star: Angle, delta_star: Angle
) -> tuple[float, Angle]: ...
def planet_stars_in_line(
    alpha_list: list[Angle] | tuple[Angle, ...],
    delta_list: list[Angle] | tuple[Angle, ...],
    alpha_star1: Angle,
    delta_star1: Angle,
    alpha_star2: Angle,
    delta_star2: Angle,
) -> float: ...
def straight_line(
    alpha1: Angle, delta1: Angle, alpha2: Angle, delta2: Angle, alpha3: Angle, delta3: Angle
) -> tuple[Angle, Angle]: ...
def circle_diameter(alpha1: Angle, delta1: Angle, alpha2: Angle, delta2: Angle, alpha3: Angle, delta3: Angle) -> Angle: ...
def vsop_pos(
    epoch: Epoch, vsop_l: list[list[list[float]]], vsop_b: list[list[list[float]]], vsop_r: list[list[list[float]]]
) -> tuple[Angle, Angle, float]: ...
def geometric_vsop_pos(
    epoch: Epoch,
    vsop_l: list[list[list[float]]],
    vsop_b: list[list[list[float]]],
    vsop_r: list[list[list[float]]],
    tofk5: bool | None = True,
) -> tuple[Angle, Angle, float]: ...
def apparent_vsop_pos(
    epoch: Epoch,
    vsop_l: list[list[list[float]]],
    vsop_b: list[list[list[float]]],
    vsop_r: list[list[list[float]]],
    nutation: bool | None = True,
) -> tuple[Angle, Angle, float]: ...
def apparent_position(epoch: Epoch, alpha: Angle, delta: Angle, sun_lon: Angle) -> tuple[Angle, Angle]: ...
def orbital_equinox2equinox(epoch0: Epoch, epoch: Epoch, i0: Angle, arg0: Angle, lon0: Angle) -> tuple[Angle, Angle, Angle]: ...
def kepler_equation(eccentricity: float, mean_anomaly: Angle) -> tuple[Angle, Angle]: ...
def orbital_elements(
    epoch: Epoch, parameters1: list[list[float]], parameters2: list[list[float]]
) -> tuple[Angle, float, float, Angle, Angle, Angle]: ...
def velocity(r: float, a: float) -> float: ...
def velocity_perihelion(e: float, a: float) -> float: ...
def velocity_aphelion(e: float, a: float) -> float: ...
def length_orbit(e: float, a: float) -> float: ...
def passage_nodes_elliptic(omega: Angle, e: float, a: float, t: Epoch, ascending: bool | None = True) -> tuple[Epoch, float]: ...
def passage_nodes_parabolic(omega: Angle, q: float, t: Epoch, ascending: bool | None = True) -> tuple[Epoch, float]: ...
def phase_angle(sun_dist: float, earth_dist: float, sun_earth_dist: float) -> Angle: ...
def illuminated_fraction(sun_dist: float, earth_dist: float, sun_earth_dist: float) -> float: ...
def main() -> None: ...



"""Symbol Table Construction Pass for the Jac compiler.

This pass builds the hierarchical symbol table structure for the entire program by:

1. Creating symbol tables for each scope in the program (modules, archetypes, abilities, blocks)
2. Establishing parent-child relationships between nested scopes
3. Registering symbols for various language constructs:
   - Global variables and imports
   - Archetypes (objects, nodes, edges, walkers) and their members
   - Abilities (methods and functions) and their parameters
   - Enums and their values
   - Local variables in various block scopes

4. Adding special symbols like 'self' and 'super' in appropriate contexts
5. Maintaining scope boundaries for proper symbol resolution

The symbol table is a fundamental data structure that enables name resolution,
type checking, and semantic analysis throughout the compilation process.
"""

import jaclang.pycore.unitree as uni
from jaclang.pycore.constant import SymbolAccess, Tokens
from jaclang.pycore.passes.uni_pass import UniPass
from jaclang.pycore.unitree import UniScopeNode


class SymTabBuildPass(UniPass):
    """Jac Symbol table build pass."""

    def before_pass(self) -> None:
        """Before pass."""
        self.cur_sym_tab: list[UniScopeNode] = []

    def push_scope_and_link(self, key_node: uni.UniScopeNode) -> None:
        """Push scope."""
        if not len(self.cur_sym_tab):
            self.cur_sym_tab.append(key_node)
        else:
            self.cur_sym_tab.append(self.cur_scope.link_kid_scope(key_node=key_node))

    def pop_scope(self) -> UniScopeNode:
        """Pop scope."""
        return self.cur_sym_tab.pop()

    @property
    def cur_scope(self) -> UniScopeNode:
        """Return current scope."""
        return self.cur_sym_tab[-1]

    def find_python_scope_node_of(self, node: uni.UniNode) -> UniScopeNode | None:
        """Find scope node of a given node."""
        scope_types = uni.UniScopeNode.get_python_scoping_nodes()
        while node.parent:
            if isinstance(node.parent, scope_types):
                return node.parent
            node = node.parent
        return None

    def _bind_import_path_symbols(self, module_path: uni.ModulePath) -> None:
        """Create symbols for Name nodes in a module path."""
        if module_path.path:
            for n in module_path.path:
                if isinstance(n, uni.Name):
                    n.sym = n.create_symbol(
                        access=SymbolAccess.PUBLIC,
                        imported=True,
                    )

    def enter_module(self, node: uni.Module) -> None:
        self.push_scope_and_link(node)

    def exit_module(self, node: uni.Module) -> None:
        self.pop_scope()

    def exit_global_vars(self, node: uni.GlobalVars) -> None:
        for i in node.assignments:
            for j in i.target:
                if isinstance(j, uni.AstSymbolNode):
                    j.sym_tab.def_insert(j, access_spec=node, single_decl="global var")
                else:
                    self.ice("Expected name type for global vars")

    def exit_assignment(self, node: uni.Assignment) -> None:
        for i in node.target:
            if isinstance(i, uni.AstSymbolNode):
                # Handle -> [a, b] = ...
                # TODO: This can be recursive for nested structures
                # example: [a, b, (c, d['e'], [f, g])] = [1, 2, (3, 4, [5, 6])]
                if isinstance(i, uni.ListVal):
                    for elem in i.values:
                        if isinstance(elem, uni.AstSymbolNode):
                            if (
                                sym := elem.sym_tab.lookup(elem.sym_name, deep=False)
                            ) is None:
                                elem.sym_tab.def_insert(elem, single_decl="local var")
                            else:
                                sym.add_use(elem.name_spec)
                # Case 2: a = b # NOTE: we're not considering the fact that the node could also be a complex
                # expression like foo.bar() and assuming that it'll be just a name, this needs to be fixed.
                elif (sym := i.sym_tab.lookup(i.sym_name, deep=False)) is None:
                    i.sym_tab.def_insert(i, single_decl="local var")
                else:
                    sym.add_use(i.name_spec)

    def exit_binary_expr(self, node: uni.BinaryExpr) -> None:
        """Handle walrus operator (:=) assignments."""
        from jaclang.pycore.constant import Tokens as Tok

        if not (isinstance(node.op, uni.Token) and node.op.name == Tok.WALRUS_EQ):
            return

        # The left side of walrus operator is the variable being assigned
        if isinstance(node.left, uni.Name):
            if (
                sym := node.left.sym_tab.lookup(node.left.sym_name, deep=False)
            ) is None:
                node.left.sym_tab.def_insert(node.left, single_decl="walrus var")
            else:
                sym.add_use(node.left.name_spec)

    def enter_test(self, node: uni.Test) -> None:
        self.push_scope_and_link(node)
        import unittest

        for i in [j for j in dir(unittest.TestCase()) if j.startswith("assert")]:
            node.sym_tab.def_insert(
                uni.Name.gen_stub_from_node(node, i, set_name_of=node)
            )

    def exit_test(self, node: uni.Test) -> None:
        self.pop_scope()

    def _exit_import_absorb(self, node: uni.Import) -> None:
        sym_table_to_update = self.find_python_scope_node_of(node)
        if sym_table_to_update is None:
            return

        # Get the module from module path
        import_all_module_path_node: uni.ModulePath = node.items[0]  # type: ignore
        import_all_module_path = import_all_module_path_node.resolve_relative_path()
        module: uni.Module | None = None
        if import_all_module_path in self.prog.mod.hub:
            module = self.prog.mod.hub[import_all_module_path]
        else:
            try:
                module = self.prog.compile(
                    import_all_module_path, no_cgen=True, type_check=False
                )
            except Exception:
                return
        # create and bind symbols
        self._bind_import_path_symbols(import_all_module_path_node)
        # 1. TODO: Check if the module has __all__ defined.
        # 2. Import all public symbols from the module
        if module:
            for sym in module.names_in_scope.values():
                if sym.access != SymbolAccess.PRIVATE:
                    sym_table_to_update.def_insert(
                        sym.defn[0],
                        single_decl="import absorb",
                    )

    def exit_import(self, node: uni.Import) -> None:
        if node.is_absorb:
            return self._exit_import_absorb(node)

    def exit_module_path(self, node: uni.ModulePath) -> None:
        if node.alias:
            node.alias.sym_tab.def_insert(node.alias, single_decl="import")
        elif (
            node.path
            and not node.is_import_from
            and node.parent_of_type(uni.Import)
            and not (
                node.parent_of_type(uni.Import).from_loc
                and node.parent_of_type(uni.Import).is_jac
            )
        ):
            # Only process if first element is a Name (not a String literal)
            if isinstance(node.path[0], uni.Name):
                node.path[0].sym_tab.def_insert(node.path[0])
        else:
            pass  # Need to support pythonic import symbols with dots in it

        # There will be symbols for
        # import from math {sqrt}  <- math will have a symbol but no symtab entry
        # import math as m  <- m will have a symbol and symtab entry
        if node.path and (node.is_import_from or (node.alias)):
            self._bind_import_path_symbols(node)

    def exit_module_item(self, node: uni.ModuleItem) -> None:
        # Check Name first (since Name is a subclass of Token)
        if isinstance(node.name, uni.Name):
            # Regular named import
            sym_node = node.alias or node.name
            sym_node.sym_tab.def_insert(sym_node, single_decl="import")
            if node.alias:
                # create symbol for module item
                node.name.sym = node.name.create_symbol(
                    access=SymbolAccess.PUBLIC,
                    imported=True,
                )
        elif isinstance(node.name, uni.Token) and node.alias:
            sym_node = node.alias
            sym_node.sym_tab.def_insert(sym_node, single_decl="import")

    def enter_archetype(self, node: uni.Archetype) -> None:
        self.push_scope_and_link(node)
        assert node.parent_scope is not None
        node.parent_scope.def_insert(node, access_spec=node, single_decl="archetype")

    def exit_archetype(self, node: uni.Archetype) -> None:
        self.pop_scope()

    def enter_ability(self, node: uni.Ability) -> None:
        self.push_scope_and_link(node)
        assert node.parent_scope is not None
        node.parent_scope.def_insert(node, access_spec=node, single_decl="ability")
        if node.is_method:
            node.sym_tab.def_insert(uni.Name.gen_stub_from_node(node, "self"))
            node.sym_tab.def_insert(
                uni.Name.gen_stub_from_node(
                    node, "super", set_name_of=node.method_owner
                )
            )

    def exit_ability(self, node: uni.Ability) -> None:
        self.pop_scope()

    def enter_impl_def(self, node: uni.ImplDef) -> None:
        self.push_scope_and_link(node)
        assert node.parent_scope is not None
        node.parent_scope.def_insert(node, single_decl="impl")

    def exit_impl_def(self, node: uni.ImplDef) -> None:
        self.pop_scope()

    def enter_sem_def(self, node: uni.SemDef) -> None:
        self.push_scope_and_link(node)
        assert node.parent_scope is not None
        node.parent_scope.def_insert(node, single_decl="sem")

    def exit_sem_def(self, node: uni.SemDef) -> None:
        self.pop_scope()

    def enter_enum(self, node: uni.Enum) -> None:
        self.push_scope_and_link(node)
        assert node.parent_scope is not None
        node.parent_scope.def_insert(node, access_spec=node, single_decl="enum")

    def enter_has_var(self, node: uni.HasVar) -> None:
        if isinstance(node.parent, uni.ArchHas):
            node.sym_tab.def_insert(
                node, single_decl="has var", access_spec=node.parent
            )

    def enter_param_var(self, node: uni.ParamVar) -> None:
        node.sym_tab.def_insert(node, single_decl="param")

    def exit_atom_trailer(self, node: uni.AtomTrailer) -> None:
        """Handle attribute access for self member assignments."""
        if not self._is_self_member_assignment(node):
            return

        chain = node.as_attr_list
        ability = node.find_parent_of_type(uni.Ability)

        # Register the attribute in the archetype's symbol table
        # Example: self.attr = value → add 'attr' to archetype.sym_tab
        if ability and ability.method_owner:
            archetype = ability.method_owner
            if isinstance(archetype, uni.Archetype):
                archetype.sym_tab.def_insert(chain[1], access_spec=archetype)

    def _is_self_member_assignment(self, node: uni.AtomTrailer) -> bool:
        """Check if the node represents a simple `self.attr = value` assignment."""
        # Must be inside an assignment as the target
        if not (node.parent and isinstance(node.parent, uni.Assignment)):
            return False

        if node != node.parent.target[0]:  # TODO: Support multiple assignment targets
            return False

        chain = node.as_attr_list

        # Must be a direct self attribute (no nested attributes)
        if len(chain) != 2 or chain[0].sym_name != "self":
            return False

        # Must be inside a non-static, non-class instance method
        ability = node.find_parent_of_type(uni.Ability)
        return (
            ability is not None
            and ability.is_method
            and not ability.is_static
            and not ability.is_cls_method
        )

    def exit_enum(self, node: uni.Enum) -> None:
        self.pop_scope()

    def enter_typed_ctx_block(self, node: uni.TypedCtxBlock) -> None:
        self.push_scope_and_link(node)

    def exit_typed_ctx_block(self, node: uni.TypedCtxBlock) -> None:
        self.pop_scope()

    def enter_if_stmt(self, node: uni.IfStmt) -> None:
        self.push_scope_and_link(node)

    def exit_if_stmt(self, node: uni.IfStmt) -> None:
        self.pop_scope()

    def enter_else_if(self, node: uni.ElseIf) -> None:
        self.push_scope_and_link(node)

    def exit_else_if(self, node: uni.ElseIf) -> None:
        self.pop_scope()

    def enter_else_stmt(self, node: uni.ElseStmt) -> None:
        self.push_scope_and_link(node)

    def exit_else_stmt(self, node: uni.ElseStmt) -> None:
        self.pop_scope()

    def enter_try_stmt(self, node: uni.TryStmt) -> None:
        self.push_scope_and_link(node)

    def exit_try_stmt(self, node: uni.TryStmt) -> None:
        self.pop_scope()

    def enter_except(self, node: uni.Except) -> None:
        self.push_scope_and_link(node)
        if node.name:
            node.sym_tab.def_insert(node.name, single_decl="local var")

    def exit_except(self, node: uni.Except) -> None:
        self.pop_scope()

    def enter_finally_stmt(self, node: uni.FinallyStmt) -> None:
        self.push_scope_and_link(node)

    def exit_finally_stmt(self, node: uni.FinallyStmt) -> None:
        self.pop_scope()

    def enter_iter_for_stmt(self, node: uni.IterForStmt) -> None:
        self.push_scope_and_link(node)

    def exit_iter_for_stmt(self, node: uni.IterForStmt) -> None:
        self.pop_scope()

    def _def_insert_unpacking(self, node: uni.Expr, sym_tab: UniScopeNode) -> None:
        """Recursively define symbols in unpacking expressions."""
        if isinstance(node, uni.Name):
            sym_tab.def_insert(node, single_decl="iterator")
        elif isinstance(node, (uni.TupleVal, uni.ListVal)):
            for target_var in node.values:
                if isinstance(target_var, uni.Expr):
                    self._def_insert_unpacking(target_var, sym_tab)
        elif isinstance(node, uni.UnaryExpr) and node.op.name == Tokens.STAR_MUL:
            self._def_insert_unpacking(node.operand, sym_tab)

    def enter_in_for_stmt(self, node: uni.InForStmt) -> None:
        self.push_scope_and_link(node)
        self._def_insert_unpacking(node.target, node.sym_tab)

    def exit_in_for_stmt(self, node: uni.InForStmt) -> None:
        self.pop_scope()

    def enter_while_stmt(self, node: uni.WhileStmt) -> None:
        self.push_scope_and_link(node)

    def exit_while_stmt(self, node: uni.WhileStmt) -> None:
        self.pop_scope()

    def enter_with_stmt(self, node: uni.WithStmt) -> None:
        self.push_scope_and_link(node)

    def exit_with_stmt(self, node: uni.WithStmt) -> None:
        self.pop_scope()

    def exit_expr_as_item(self, node: uni.ExprAsItem) -> None:
        if node.alias and isinstance(node.alias, uni.Name):
            node.alias.sym_tab.def_insert(node.alias, single_decl="context var")

    def enter_lambda_expr(self, node: uni.LambdaExpr) -> None:
        self.push_scope_and_link(node)

    def exit_lambda_expr(self, node: uni.LambdaExpr) -> None:
        self.pop_scope()

    def enter_list_compr(self, node: uni.ListCompr) -> None:
        self.push_scope_and_link(node)
        for i in node.compr:
            self._def_insert_unpacking(i.target, node.sym_tab)

    def exit_list_compr(self, node: uni.ListCompr) -> None:
        self.pop_scope()

    def enter_set_compr(self, node: uni.SetCompr) -> None:
        self.push_scope_and_link(node)
        for i in node.compr:
            self._def_insert_unpacking(i.target, node.sym_tab)

    def exit_set_compr(self, node: uni.SetCompr) -> None:
        self.pop_scope()

    def enter_gen_compr(self, node: uni.GenCompr) -> None:
        self.push_scope_and_link(node)
        for i in node.compr:
            self._def_insert_unpacking(i.target, node.sym_tab)

    def exit_gen_compr(self, node: uni.GenCompr) -> None:
        self.pop_scope()

    def enter_dict_compr(self, node: uni.DictCompr) -> None:
        self.push_scope_and_link(node)
        for i in node.compr:
            self._def_insert_unpacking(i.target, node.sym_tab)

    def exit_dict_compr(self, node: uni.DictCompr) -> None:
        self.pop_scope()

    def enter_match_case(self, node: uni.MatchCase) -> None:
        self.push_scope_and_link(node)

    def exit_match_case(self, node: uni.MatchCase) -> None:
        self.pop_scope()



def sign_jwt_bearer_assertion(
    key, issuer, audience, subject=None, issued_at=None, expires_at=None, claims=None, header=None, **kwargs
): ...
def client_secret_jwt_sign(client_secret, client_id, token_endpoint, alg: str = "HS256", claims=None, **kwargs): ...
def private_key_jwt_sign(private_key, client_id, token_endpoint, alg: str = "RS256", claims=None, **kwargs): ...



from _typeshed import Incomplete
from typing import type_check_only

from reportlab.graphics.shapes import Rect
from reportlab.graphics.widgetbase import Widget
from reportlab.lib.validators import Validator
from reportlab.platypus.flowables import Flowable

__all__ = ["QrCodeWidget"]

@type_check_only
class _isLevel(Validator):
    def test(self, x): ...

isLevel: _isLevel

@type_check_only
class _isUnicodeOrQRList(Validator):
    def test(self, x): ...
    def normalize(self, x): ...

isUnicodeOrQRList: _isUnicodeOrQRList

class SRect(Rect):
    def __init__(self, x, y, width, height, fillColor=...) -> None: ...

class QrCodeWidget(Widget):
    codeName: str
    x: int
    y: int
    barFillColor: Incomplete
    barStrokeColor: Incomplete
    barStrokeWidth: int
    barHeight: Incomplete
    barWidth: Incomplete
    barBorder: int
    barLevel: str
    qrVersion: Incomplete
    value: Incomplete
    def __init__(self, value: str = "Hello World", **kw) -> None: ...
    def addData(self, value) -> None: ...
    def draw(self): ...

class QrCode(Flowable):
    height: Incomplete
    width: Incomplete
    qrBorder: int
    qrLevel: str
    qrVersion: Incomplete
    value: Incomplete
    qr: Incomplete
    def __init__(self, value=None, **kw) -> None: ...
    def addData(self, value) -> None: ...
    def draw(self) -> None: ...
    def rect(self, x, y, w, h) -> None: ...



import re
from collections.abc import Iterable
from typing import AnyStr, Generic

from .spawnbase import SpawnBase, _CompiledRePattern, _CompiledStringPattern, _Searcher

class searcher_string(Generic[AnyStr]):
    eof_index: int
    timeout_index: int
    longest_string: int
    def __init__(self, strings: Iterable[_CompiledStringPattern[AnyStr]]) -> None: ...
    match: AnyStr
    start: int
    end: int
    def search(self, buffer: AnyStr, freshlen: int, searchwindowsize: int | None = None): ...

class searcher_re(Generic[AnyStr]):
    eof_index: int
    timeout_index: int
    def __init__(self, patterns: Iterable[_CompiledRePattern[AnyStr]]) -> None: ...
    match: re.Match[AnyStr]
    start: int
    end: int
    def search(self, buffer: AnyStr, freshlen: int, searchwindowsize: int | None = None): ...

class Expecter(Generic[AnyStr]):
    spawn: SpawnBase[AnyStr]
    searcher: _Searcher[AnyStr]
    searchwindowsize: int | None
    lookback: _Searcher[AnyStr] | int | None
    def __init__(self, spawn: SpawnBase[AnyStr], searcher: _Searcher[AnyStr], searchwindowsize: int | None = -1) -> None: ...
    def do_search(self, window: AnyStr, freshlen: int) -> int: ...
    def existing_data(self) -> int: ...
    def new_data(self, data: AnyStr) -> int: ...
    def eof(self, err: object = None) -> int: ...
    def timeout(self, err: object = None) -> int: ...
    def errored(self) -> None: ...
    def expect_loop(self, timeout: float | None = -1) -> int: ...



from _typeshed import Incomplete
from collections.abc import Iterable

from . import base

class Filter(base.Filter[dict[str, Incomplete]]):
    encoding: str | None
    def __init__(self, source: Iterable[dict[str, Incomplete]], encoding: str | None) -> None: ...



from typing import TypedDict, type_check_only

# Unused in this module, but imported in multiple submodules.
@type_check_only
class _EncodedRLE(TypedDict):  # noqa: Y049
    size: list[int]
    counts: str | bytes



from collections.abc import Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["generate_adjlist", "write_adjlist", "parse_adjlist", "read_adjlist"]

def generate_adjlist(G: Graph[_Node], delimiter: str = " ") -> Generator[str, None, None]: ...
def write_adjlist(G: Graph[_Node], path, comments: str = "#", delimiter: str = " ", encoding: str = "utf-8") -> None: ...
@_dispatchable
def parse_adjlist(lines, comments: str = "#", delimiter=None, create_using=None, nodetype=None): ...
@_dispatchable
def read_adjlist(path, comments: str = "#", delimiter=None, create_using=None, nodetype=None, encoding: str = "utf-8"): ...



from __future__ import annotations

from wtforms import Field, FieldList, Form, FormField, SelectField, StringField
from wtforms.widgets import Input, ListWidget, Option, Select, TableWidget, TextArea

# more specific widgets should only work on more specific fields
Field(widget=Input())
Field(widget=TextArea())  # type: ignore
Field(widget=Select())  # type: ignore

# less specific widgets are fine, even if they're often not what you want
StringField(widget=Input())
StringField(widget=TextArea())

SelectField(widget=Input(), option_widget=Input())
SelectField(widget=Select(), option_widget=Option())
# a more specific type other than Option widget is not allowed
SelectField(widget=Select(), option_widget=TextArea())  # type: ignore

# we should be able to pass Field() even though it wants an unbound_field
# this gets around __new__ not working in type checking
FieldList(Field(), widget=Input())
FieldList(Field(), widget=ListWidget())

FormField(Form, widget=Input())
FormField(Form, widget=TableWidget())



from .npipeconn import NpipeHTTPAdapter as NpipeHTTPAdapter
from .npipesocket import NpipeSocket as NpipeSocket
from .sshconn import SSHHTTPAdapter as SSHHTTPAdapter
from .unixconn import UnixHTTPAdapter as UnixHTTPAdapter



"""ECMAScript/JavaScript AST generation for Jac.

This package provides ECMAScript AST generation capabilities following the ESTree
specification, allowing Jac code to be transpiled to JavaScript/ECMAScript.
"""

from jaclang.compiler.passes.ecmascript.esast_gen_pass import EsastGenPass
from jaclang.compiler.passes.ecmascript.estree import (
    Declaration,
    Expression,
    Pattern,
    Program,
    Statement,
    es_node_to_dict,
)

__all__ = [
    "EsastGenPass",
    "Expression",
    "Declaration",
    "Pattern",
    "Program",
    "Statement",
    "es_node_to_dict",
]



from typing import Any

from dateparser.search.detection import BaseLanguageDetector

class FullTextLanguageDetector(BaseLanguageDetector):
    languages: Any
    language_unique_chars: Any
    language_chars: Any
    def __init__(self, languages) -> None: ...
    def get_unique_characters(self, settings) -> None: ...
    def character_check(self, date_string, settings) -> None: ...



from _typeshed import ReadableBuffer, Unused
from typing import Final
from xml.sax import ErrorHandler as _ErrorHandler, _Source, xmlreader
from xml.sax.handler import _ContentHandlerProtocol

from .expatreader import DefusedExpatParser

__origin__: Final = "xml.sax"

def parse(
    source: xmlreader.InputSource | _Source,
    handler: _ContentHandlerProtocol,
    errorHandler: _ErrorHandler = ...,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> None: ...
def parseString(
    string: ReadableBuffer,
    handler: _ContentHandlerProtocol,
    errorHandler: _ErrorHandler = ...,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> None: ...
def make_parser(parser_list: Unused = []) -> DefusedExpatParser: ...



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["greedy_modularity_communities", "naive_greedy_modularity_communities"]

@_dispatchable
def greedy_modularity_communities(
    G: Graph[_Node], weight: str | None = None, resolution: float | None = 1, cutoff: int | None = 1, best_n: int | None = None
) -> list[set[Incomplete]] | list[frozenset[Incomplete]]: ...
@_dispatchable
def naive_greedy_modularity_communities(G: Graph[_Node], resolution: float = 1, weight: str | None = None): ...



from _typeshed import ReadableBuffer
from collections.abc import Iterable
from typing import Final, SupportsBytes, SupportsIndex

from . import connections as connections, constants as constants, converters as converters, cursors as cursors
from .constants import FIELD_TYPE as FIELD_TYPE
from .err import (
    DatabaseError as DatabaseError,
    DataError as DataError,
    Error as Error,
    IntegrityError as IntegrityError,
    InterfaceError as InterfaceError,
    InternalError as InternalError,
    MySQLError as MySQLError,
    NotSupportedError as NotSupportedError,
    OperationalError as OperationalError,
    ProgrammingError as ProgrammingError,
    Warning as Warning,
)
from .times import (
    Date as Date,
    DateFromTicks as DateFromTicks,
    Time as Time,
    TimeFromTicks as TimeFromTicks,
    Timestamp as Timestamp,
    TimestampFromTicks as TimestampFromTicks,
)

VERSION: Final[tuple[str | int, ...]]
VERSION_STRING: Final[str]
version_info: tuple[int, int, int, str, int]
__version__: str

def get_client_info() -> str: ...
def install_as_MySQLdb() -> None: ...

threadsafety: int
apilevel: str
paramstyle: str

class DBAPISet(frozenset[int]):
    def __ne__(self, other: object) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...

STRING: DBAPISet
BINARY: DBAPISet
NUMBER: DBAPISet
DATE: DBAPISet
TIME: DBAPISet
TIMESTAMP: DBAPISet
DATETIME: DBAPISet
ROWID: DBAPISet

def Binary(x: Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer) -> bytes: ...
def thread_safe() -> bool: ...

NULL: str

Connect = connections.Connection
connect = connections.Connection
Connection = connections.Connection

__all__ = [
    "BINARY",
    "Binary",
    "Connect",
    "Connection",
    "DATE",
    "Date",
    "Time",
    "Timestamp",
    "DateFromTicks",
    "TimeFromTicks",
    "TimestampFromTicks",
    "DataError",
    "DatabaseError",
    "Error",
    "FIELD_TYPE",
    "IntegrityError",
    "InterfaceError",
    "InternalError",
    "MySQLError",
    "NULL",
    "NUMBER",
    "NotSupportedError",
    "DBAPISet",
    "OperationalError",
    "ProgrammingError",
    "ROWID",
    "STRING",
    "TIME",
    "TIMESTAMP",
    "Warning",
    "apilevel",
    "connect",
    "connections",
    "constants",
    "converters",
    "cursors",
    "get_client_info",
    "paramstyle",
    "threadsafety",
    "version_info",
    "install_as_MySQLdb",
    "__version__",
]



from __future__ import annotations

from collections.abc import Iterator
from typing import Generic, TypeVar
from typing_extensions import assert_type

x: list[int] = []
assert_type(list(reversed(x)), "list[int]")


class MyReversible:
    def __iter__(self) -> Iterator[str]:
        yield "blah"

    def __reversed__(self) -> Iterator[str]:
        yield "blah"


assert_type(list(reversed(MyReversible())), "list[str]")


_T = TypeVar("_T")


class MyLenAndGetItem(Generic[_T]):
    def __len__(self) -> int:
        return 0

    def __getitem__(self, item: int) -> _T:
        raise KeyError


len_and_get_item: MyLenAndGetItem[int] = MyLenAndGetItem()
assert_type(list(reversed(len_and_get_item)), "list[int]")
